<?php

namespace Relay;

/**
 * A collection of Redis data types.
 */
enum KeyType: int
{
    /**
     * @see Relay::REDIS_NOT_FOUND
     */
    case NotFound = Relay::REDIS_NOT_FOUND;

    /**
     * @see Relay::REDIS_STRING
     */
    case String = Relay::REDIS_STRING;

    /**
     * @see Relay::REDIS_SET
     */
    case Set = Relay::REDIS_SET;

    /**
     * @see Relay::REDIS_LIST
     */
    case List = Relay::REDIS_LIST;

    /**
     * @see Relay::REDIS_ZSET
     */
    case Zset = Relay::REDIS_ZSET;

    /**
     * @see Relay::REDIS_HASH
     */
    case Hash = Relay::REDIS_HASH;

    /**
     * @see Relay::REDIS_STREAM
     */
    case Stream = Relay::REDIS_STREAM;
}

/**
 * Relay client.
 */
class Relay
{
    /**
     * Relay's version.
     *
     * @var string
     */
    public const VERSION = "0.9.0";

    /**
     * Relay's version.
     *
     * @var string
     */
    public const Version = "0.9.0";

    /**
     * Integer representing no compression algorithm.
     *
     * @var int
     */
    public const COMPRESSION_NONE = 0;

    /**
     * Integer representing the LZF compression algorithm.
     *
     * @var int
     */
    public const COMPRESSION_LZF = 1;

    /**
     * Integer representing the Zstandard compression algorithm.
     *
     * @var int
     */
    public const COMPRESSION_ZSTD = 2;

    /**
     * Integer representing the LZ4 compression algorithm.
     *
     * @var int
     */
    public const COMPRESSION_LZ4 = 3;

    /**
     * Integer representing no serializer.
     *
     * @var int
     */
    public const SERIALIZER_NONE = 0;

    /**
     * Integer representing the PHP serializer.
     *
     * @var int
     */
    public const SERIALIZER_PHP = 1;

    /**
     * Integer representing the igbinary serializer.
     *
     * @var int
     */
    public const SERIALIZER_IGBINARY = 2;

    /**
     * Integer representing the MessagePack serializer.
     *
     * @var int
     */
    public const SERIALIZER_MSGPACK = 3;

    /**
     * Integer representing the JSON serializer.
     *
     * @var int
     */
    public const SERIALIZER_JSON = 4;

    /**
     * Integer representing the atomic mode.
     *
     * @see Relay::getMode()
     * @var int
     */
    public const ATOMIC = 0x00;

    /**
     * Integer representing the pipeline mode.
     *
     * @see Relay::getMode()
     * @var int
     */
    public const PIPELINE = 0x02;

    /**
     * Integer representing the `MULTI` mode.
     *
     * @see Relay::getMode()
     * @var int
     */
    public const MULTI = 0x01;

    /**
     * Integer representing we're SUBSCRIBED.  Note that this constant can
     * only really be accessed when `true` is passed to `getMask()` telling
     * relay to return the complete bitmasked mode.
     *
     * @see Relay::getMode()
     * @var int
     **/
    public const SUBSCRIBED = 0x04;

    /**
     * Integer representing the prefix option.
     *
     * @var int
     */
    public const OPT_PREFIX = 2;

    /**
     * Integer representing the read timeout option.
     *
     * @var int
     */
    public const OPT_READ_TIMEOUT = 3;

    /**
     * Integer representing the maximum retries option.
     *
     * @var int
     */
    public const OPT_MAX_RETRIES = 11;

    /**
     * Integer representing the backoff algorithm.
     *
     * @var int
     */
    public const OPT_BACKOFF_ALGORITHM = 12;

    /**
     * Toggle TCP_KEEPALIVE on a connection
     *
     * @var int
     */
    public const OPT_TCP_KEEPALIVE = 6;

    /**
     * Integer representing the default backoff algorithm.
     *
     * @var int
     */
    public const BACKOFF_ALGORITHM_DEFAULT = 0;

    /**
     * Integer representing the decorrelated jitter backoff algorithm.
     *
     * @var int
     */
    public const BACKOFF_ALGORITHM_DECORRELATED_JITTER = 1;

    /**
     * Integer representing the full jitter backoff algorithm.
     *
     * @var int
     */
    public const BACKOFF_ALGORITHM_FULL_JITTER = 2;

    /**
     * Integer representing the base for backoff computation.
     *
     * @var int
     */
    public const OPT_BACKOFF_BASE = 13;

    /**
     * Integer representing the backoff time cap.
     *
     * @var int
     */
    public const OPT_BACKOFF_CAP = 14;

    /**
     * Integer representing the PhpRedis compatibility mode option.
     *
     * Enabled by default. Disabling will cause Relay to:
     * 1. Return `null` when a key doesn't exist, instead of `false`
     * 2. Throw exceptions when an error occurs, instead of returning `false`
     *
     * @var int
     */
    public const OPT_PHPREDIS_COMPATIBILITY = 100;

    /**
     * Integer representing the serializer option.
     *
     * @var int
     */
    public const OPT_SERIALIZER = 1;

    /**
     * Integer representing the compression option.
     *
     * @var int
     */
    public const OPT_COMPRESSION = 7;

    /**
     * Integer representing the compression level option.
     *
     * @var int
     */
    public const OPT_COMPRESSION_LEVEL = 9;

    /**
     * Integer representing the reply literal option.
     *
     * @var int
     */
    public const OPT_REPLY_LITERAL = 8;

    /**
     * Integer representing the null-multi-bulk-as-null option.
     *
     * @var int
     */
    public const OPT_NULL_MULTIBULK_AS_NULL = 10;

    /**
     * Integer representing the throw-on-error option.
     *
     * Disabled by default. When enabled, Relay will throw exceptions when errors occur.
     *
     * @var int
     */
    public const OPT_THROW_ON_ERROR = 105;

    /**
     * Integer representing Relay’s invalidation option.
     *
     * Enabled by default. When disabled will prevent Relay from
     * performing instantaneous client-side invalidation when a key
     * is changed without waiting for Redis to send an `INVALIDATE`
     * message. The invalidation occurs only in the same FPM pool.
     *
     * @var int
     */
    public const OPT_CLIENT_INVALIDATIONS = 101;

    /**
     * Integer representing Relay’s allow patterns option.
     *
     * When set only keys matching these patterns will be cached,
     * unless they also match an `OPT_IGNORE_PATTERNS`.
     *
     * @var int
     */
    public const OPT_ALLOW_PATTERNS = 102;

    /**
     * Integer representing Relay’s ignore patterns option.
     *
     * When set keys matching these patterns will not be cached.
     *
     * @var int
     */
    public const OPT_IGNORE_PATTERNS = 103;

    /**
     * Whether use in-memory caching. Enabled by default.
     *
     * @var int
     */
    public const OPT_USE_CACHE = 104;

    /**
     * Integer representing the scan option.
     *
     * @var int
     */
    public const OPT_SCAN = 4;

    /**
     * Issue one `SCAN` command at a time, sometimes returning an empty array of results.
     *
     * @var int
     */
    public const SCAN_NORETRY = 0;

    /**
     * Retry the `SCAN` command until keys come back, or iterator of zero is returned.
     *
     * @var int
     */
    public const SCAN_RETRY = 1;

    /**
     * Prepend the set prefix to any `MATCH` pattern.
     *
     * @var int
     */
    public const SCAN_PREFIX = 2;

    /**
     * Do not prepend the set prefix to any `MATCH` pattern.
     *
     * @var int
     */
    public const SCAN_NOPREFIX = 3;

    /**
     * Redis command argument.
     *
     * @internal
     * @var string
     */
    public const BEFORE = 'BEFORE';

    /**
     * Redis command argument.
     *
     * @internal
     * @var string
     */
    public const AFTER = 'AFTER';

    /**
     * Redis command argument.
     *
     * @internal
     * @var string
     */
    public const LEFT = 'LEFT';

    /**
     * Redis command argument.
     *
     * @internal
     * @var string
     */
    public const RIGHT = 'RIGHT';

    /**
     * Integer representing "key not found".
     *
     * @see Relay::type()
     * @var int
     */
    public const REDIS_NOT_FOUND = 0;

    /**
     * Integer representing Redis `string` type.
     *
     * @see Relay::type()
     * @var int
     */
    public const REDIS_STRING = 1;

    /**
     * Integer representing Redis `set` type.
     *
     * @see Relay::type()
     * @var int
     */
    public const REDIS_SET = 2;

    /**
     * Integer representing Redis `list` type.
     *
     * @see Relay::type()
     * @var int
     */
    public const REDIS_LIST = 3;

    /**
     * Integer representing Redis `zset` type.
     *
     * @see Relay::type()
     * @var int
     */
    public const REDIS_ZSET = 4;

    /**
     * Integer representing Redis `hash` type.
     *
     * @see Relay::type()
     * @var int
     */
    public const REDIS_HASH = 5;

    /**
     * Integer representing Redis `stream` type.
     *
     * @see Relay::type()
     * @var int
     */
    public const REDIS_STREAM = 6;

    /**
     * Establishes a new connection to Redis, or re-uses already opened connection.
     *
     * @param  string|array|null  $host
     * @param  int  $port
     * @param  float  $connect_timeout
     * @param  float  $command_timeout
     * @param  array  $context
     */
    #[Attributes\Server]
    public function __construct(
        string|array|null $host = null,
        int $port = 6379,
        float $connect_timeout = 0.0,
        float $command_timeout = 0.0,
        #[\SensitiveParameter]
        array $context = [],
        int $database = 0,
    );

    /**
     * Establishes a new connection to Redis.
     * Will use `pconnect()` unless `relay.default_pconnect` is disabled.
     *
     * @param  string  $host
     * @param  int  $port
     * @param  float  $timeout
     * @param  string|null  $persistent_id
     * @param  int  $retry_interval
     * @param  float  $read_timeout
     * @param  array  $context
     * @param  int  $database
     * @return bool
     */
    #[Attributes\Server]
    public function connect(
        string $host,
        int $port = 6379,
        float $timeout = 0.0,
        ?string $persistent_id = null,
        int $retry_interval = 0,
        float $read_timeout = 0.0,
        #[\SensitiveParameter]
        array $context = [],
        int $database = 0
    ): bool;

    /**
     * Establishes a persistent connection to Redis.
     *
     * @param  string  $host
     * @param  int  $port
     * @param  float  $timeout
     * @param  string|null  $persistent_id
     * @param  int  $retry_interval
     * @param  float  $read_timeout
     * @param  array  $context
     * @param  int  $database
     * @return bool
     */
    #[Attributes\Server]
    public function pconnect(
        string $host,
        int $port = 6379,
        float $timeout = 0.0,
        ?string $persistent_id = null,
        int $retry_interval = 0,
        float $read_timeout = 0.0,
        #[\SensitiveParameter]
        array $context = [],
        int $database = 0
    ): bool;

    /**
     * Closes the current connection, unless it's persistent.
     *
     * @return bool
     */
    #[Attributes\Local]
    public function close(): bool;

    /**
     * Closes the current connection, if it's persistent.
     *
     * @return bool
     */
    #[Attributes\Local]
    public function pclose(): bool;

    /**
     * Registers a new event listener.
     *
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\Local]
    public function listen(?callable $callback): bool;

    /**
     * Registers a new `flushed` event listener.
     *
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\Local]
    public function onFlushed(?callable $callback): bool;

    /**
     * Registers a new `invalidated` event listener.
     *
     * @param  callable  $callback
     * @param  string|null  $pattern
     * @return bool
     */
    #[Attributes\Local]
    public function onInvalidated(?callable $callback, ?string $pattern = null): bool;

    /**
     * Dispatches all pending events.
     *
     * @return int|false
     */
    #[Attributes\Local]
    public function dispatchEvents(): int|false;

    /**
     * Returns a client option.
     *
     * @param  int  $option
     * @return mixed
     */
    #[Attributes\Local]
    public function getOption(int $option): mixed;

    /**
     * Returns or sets a client option.
     *
     * @param  int  $option
     * @param  mixed  $value
     * @return mixed
     */
    #[Attributes\Local]
    public function option(int $option, mixed $value = null): mixed;

    /**
     * Sets a client option.
     *
     * Relay specific options:
     *
     * - `OPT_ALLOW_PATTERNS`
     * - `OPT_IGNORE_PATTERNS`
     * - `OPT_THROW_ON_ERROR`
     * - `OPT_CLIENT_INVALIDATIONS`
     * - `OPT_PHPREDIS_COMPATIBILITY`
     *
     * Supported PhpRedis options:
     *
     * - `OPT_PREFIX`
     * - `OPT_READ_TIMEOUT`
     * - `OPT_COMPRESSION`
     * - `OPT_COMPRESSION_LEVEL`
     * - `OPT_MAX_RETRIES`
     * - `OPT_BACKOFF_ALGORITHM`
     * - `OPT_BACKOFF_BASE`
     * - `OPT_BACKOFF_CAP`
     * - `OPT_SCAN`
     * - `OPT_REPLY_LITERAL`
     * - `OPT_NULL_MULTIBULK_AS_NULL`
     *
     * @param  int  $option
     * @param  mixed  $value
     * @return bool
     */
    #[Attributes\Local]
    public function setOption(int $option, mixed $value): bool;

    /**
     * Adds ignore pattern(s). Matching keys will not be cached in memory.
     *
     * @param  string  $pattern,...
     * @return int
     */
    #[Attributes\Local]
    public function addIgnorePatterns(string ...$pattern): int;

    /**
     * Adds allow pattern(s). Only matching keys will be cached in memory.
     *
     * @param  string  $pattern,...
     * @return int
     */
    #[Attributes\Local]
    public function addAllowPatterns(string ...$pattern): int;

    /**
     * Returns the connection timeout.
     *
     * @return float|false
     */
    #[Attributes\Local]
    public function getTimeout(): float|false;

    /**
     * @alias Relay\Relay::getTimeout
     *
     * @return float|false
     */
    #[Attributes\Local]
    public function timeout(): float|false;

    /**
     * Returns the read timeout.
     *
     * @return float|false
     */
    #[Attributes\Local]
    public function getReadTimeout(): float|false;

    /**
     * @alias Relay\Relay::getReadTimeout
     *
     * @return float|false
     */
    #[Attributes\Local]
    public function readTimeout(): float|false;

    /**
     * Returns the number of bytes sent and received over the network during the Relay object's
     * lifetime, or since the last time {@link Relay::clearBytes()} was called.
     *
     * @return array
     */
    #[Attributes\Local]
    public function getBytes(): array;

    /**
     * @alias Relay\Relay::getBytes
     *
     * @return array
     */
    #[Attributes\Local]
    public function bytes(): array;

    /**
     * Returns the host or unix socket.
     *
     * @return string|false
     */
    #[Attributes\Local]
    public function getHost(): string|false;

    /**
     * Whether Relay is connected to Redis.
     *
     * @return bool
     */
    #[Attributes\Local]
    public function isConnected(): bool;

    /**
     * Returns the port.
     *
     * @return int|false
     */
    #[Attributes\Local]
    public function getPort(): int|false;

    /**
     * Returns the authentication information.
     * In PhpRedis compatibility mode this method returns any configured password in plain-text.
     *
     * @return mixed
     */
    #[Attributes\Local]
    public function getAuth(): mixed;

    /**
     * Returns the currently selected DB
     *
     * @return int|false
     */
    #[Attributes\Local]
    public function getDbNum(): mixed;

    /**
     * Returns the serialized value.
     *
     * @param  mixed  $value
     * @return mixed
     */
    #[Attributes\Local]
    public function _serialize(mixed $value): mixed;

    /**
     * Returns the unserialized value.
     *
     * @param  mixed  $value
     * @return mixed
     */
    #[Attributes\Local]
    public function _unserialize(mixed $value): mixed;

    /**
     * Compress data with Relay's currently configured compression algorithm.
     *
     * @param  string  $value
     * @return string
     */
    #[Attributes\Local]
    public function _compress(string $value): string;

    /**
     * Uncompress data with Relay's currently configured compression algorithm.
     *
     * @param  string  $value
     * @return string
     */
    #[Attributes\Local]
    public function _uncompress(string $value): string;

    /**
     * Returns the serialized and compressed value.
     *
     * @param  mixed  $value
     * @return string
     */
    #[Attributes\Local]
    public function _pack(mixed $value): string;

    /**
     * Returns the unserialized and decompressed value.
     *
     * @param  mixed  $value
     * @return mixed
     */
    #[Attributes\Local]
    public function _unpack(mixed $value): mixed;

    /**
     * Returns the value with the prefix.
     *
     * @param  mixed  $value
     * @return string
     */
    #[Attributes\Local]
    public function _prefix(mixed $value): string;

    /**
     * Returns the last error message, if any.
     *
     * @return string|null
     */
    #[Attributes\Local]
    public function getLastError(): string|null;

    /**
     * Clears the last error that is set, if any.
     *
     * @return bool
     */
    #[Attributes\Local]
    public function clearLastError(): bool;

    /**
     * Returns the connection's endpoint identifier.
     *
     * @return string|false
     */
    #[Attributes\Local]
    public function endpointId(): string|false;

    /**
     * @alias Relay\Relay::endpointId
     *
     * @return string|false
     */
    public function getPersistentID(): string|false;

    /**
     * Returns a unique representation of the underlying socket connection identifier.
     *
     * @return string|false
     */
    #[Attributes\Local]
    public function socketId(): string|false;

    /**
     * Returns statistics about Relay.
     *
     * - `usage.total_requests`: The total number of requests we've seen
     * - `usage.active_requests`: The number of requests currently in-flight
     * - `usage.max_active_requests`: The most concurrent in-flight requests we've seen
     * - `usage.free_epoch_records`: The estimated number of free epoch reclamation records
     *
     * - `stats.requests`: The total number of requests the cache has received
     * - `stats.misses`: Requests where we had to ask Redis for a value
     * - `stats.hits`: Requests where we did not have to ask redis for the value
     * - `stats.dirty_skips`: The number of times Relay has skipped an entire database because it was dirty.
     * - `stats.errors`: How many times a 'severe' error occurs (presently this is only incremented if we get a `null` response from hiredis)
     * - `stats.empty`: How many times we've run out of free requests (indicating the size of the ring buffers should be increased)
     * - `stats.oom`: The number of times we've run out of memory
     * - `stats.ops_per_sec`: The number of commands processed per second
     * - `stats.bytes_sent`: The number of bytes Relay has written to the network
     * - `stats.bytes_received`: The number of bytes Relay has read from the network
     * - `stats.command_usec`: Deprecated
     * - `stats.rinit_usec`: The number of microseconds Relay has spent in request initialization
     * - `stats.rshutdown_usec`: The number of microseconds Relay has spent in request shutdown
     * - `stats.sigio_usec`: The number of microseconds Relay has spent in its SIGIO handler
     *
     * - `memory.total`: The total bytes of allocated memory
     * - `memory.limit`: The capped number of bytes Relay has available to use
     * - `memory.active`: The total amount of memory mapped into the allocator
     * - `memory.used`: The amount of memory pointing to live objects including metadata
     *
     * - `endpoints.*.redis`: Information about the connected Redis server.
     * - `endpoints.*.connections.*.keys`: The total number of cached keys for the connection.
     *
     * @return array
     */
    #[Attributes\Local]
    public static function stats(): array;

    /**
     * Returns the number of bytes allocated, or `0` in client-only mode.
     *
     * @return int
     */
    #[Attributes\Local]
    public static function maxMemory(): int;

    /**
     * Returns the number of bytes allocated, or `0` in client-only mode.
     *
     * @deprecated 0.5.0 Use `Relay:maxMemory()`
     *
     * @return int
     */
    #[Attributes\Local]
    public static function memory(): int;

    /**
     * Execute any command against Redis, without applying
     * the prefix, compression and serialization.
     *
     * @param  string  $cmd
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function rawCommand(string $cmd, mixed ...$args): mixed;

    /**
     * Select the Redis logical database having the specified zero-based numeric index.
     *
     * @param  int  $db
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function select(int $db): Relay|bool;

    /**
     * Authenticate the connection using a password or an ACL username and password.
     *
     * @param  mixed  $auth
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function auth(#[\SensitiveParameter] mixed $auth): bool;

    /**
     * The INFO command returns information and statistics about Redis in a format
     * that is simple to parse by computers and easy to read by humans.
     *
     * @see https://redis.io/commands/info
     *
     * @param  string  $sections,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function info(string ...$sections): Relay|array|false;

    /**
     * Deletes all the keys of the currently selected database.
     *
     * @param  bool|null  $sync
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function flushdb(?bool $sync = null): Relay|bool;

    /**
     * Deletes all the keys of all the existing databases, not just the currently selected one.
     *
     * @param  bool|null  $sync
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function flushall(?bool $sync = null): Relay|bool;

    /**
     * Invokes a Redis function.
     *
     * @param  string  $name
     * @param  array  $keys
     * @param  array  $argv
     * @param  callable|null  $handler
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function fcall(string $name, array $keys = [], array $argv = [], ?callable $handler = null): mixed;

    /**
     * Invokes a read-only Redis function.
     *
     * @param  string  $name
     * @param  array  $keys
     * @param  array  $argv
     * @param  callable|null  $handler
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function fcall_ro(string $name, array $keys = [], array $argv = [], callable $handler = null): mixed;

    /**
     * Calls `FUNCTION` sub-command.
     *
     * @param  string  $op
     * @param  string  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function function(string $op, string ...$args): mixed;

    /**
     * Flushes Relay's in-memory cache of all databases.
     * When given an endpoint, only that connection will be flushed.
     * When given an endpoint and database index, only that database
     * for that connection will be flushed.
     *
     * @param  string|null  $endpointId
     * @param  int|null  $db
     * @return bool
     */
    #[Attributes\Local]
    public static function flushMemory(?string $endpointId = null, int $db = null): bool;

    /**
     * Run a search query on an index, and perform aggregate
     * transformations on the results, extracting statistics etc from them.
     *
     * @param  mixed  $index
     * @param  string  $query
     * @param  array|null  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftAggregate(mixed $index, string $query, ?array $options = null): Relay|array|false;

    /**
     * Add an alias to an index.
     *
     * @param  mixed  $index
     * @param  string  $alias
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ftAliasAdd(mixed $index, string $alias): Relay|bool;

    /**
     * Remove an alias from an index.
     *
     * @param  string  $alias
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ftAliasDel(string $alias): Relay|bool;

    /**
     * Add an alias to an index.
     * If the alias is already associated with another index,
     * removes the alias association with the previous index.
     *
     * @param  mixed  $index
     * @param  string  $alias
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ftAliasUpdate(mixed $index, string $alias): Relay|bool;

    /**
     * Add a new attribute to the index.
     * Adding an attribute to the index causes any future doacument updates
     * to use the new attribute when indexing and reindexing existing documents.
     *
     * @param  mixed  $index
     * @param  array  $schema
     * @param  bool  $skipinitialscan
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ftAlter(mixed $index, array $schema, bool $skipinitialscan = false): Relay|bool;

    /**
     * Container command for get/set RediSearch configuration parameter.
     *
     * @param  string  $operation
     * @param  string  $option
     * @param  mixed  $value
     * @return Relay|array|bool
     */
    #[Attributes\RedisCommand]
    public function ftConfig(string $operation, string $option, mixed $value = null): Relay|array|bool;

    /**
     * Create an index with the given specification.
     *
     * @param  mixed  $index
     * @param  array  $schema
     * @param  array|null  $options
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ftCreate(mixed $index, array $schema, ?array $options = null): Relay|bool;

    /**
     * Container command for del/read existing cursor.
     *
     * @param  string  $operation
     * @param  mixed  $index
     * @param  mixed  $cursor
     * @param  array|null  $options
     * @return Relay|array|bool
     */
    #[Attributes\RedisCommand]
    public function ftCursor(string $operation, mixed $index, mixed $cursor, ?array $options = null): Relay|array|bool;

    /**
     * Add terms to a dictionary.
     *
     * @param  mixed  $dict
     * @param  mixed  $term
     * @param  mixed  $other_terms,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function ftDictAdd(mixed $dict, mixed $term, mixed ...$other_terms): Relay|int|false;

    /**
     * Delete terms from a dictionary.
     *
     * @param  mixed  $dict
     * @param  mixed  $term
     * @param  mixed  $other_terms,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function ftDictDel(mixed $dict, mixed $term, mixed ...$other_terms): Relay|int|false;

    /**
     * Dump all terms in the given dictionary.
     *
     * @param  mixed  $dict
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftDictDump(mixed $dict): Relay|array|false;

    /**
     * Delete an index.
     *
     * @param  mixed  $index
     * @param  bool  $dd
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ftDropIndex(mixed $index, bool $dd = false): Relay|bool;

    /**
     * Return the execution plan for a complex query.
     *
     * @param  mixed  $index
     * @param  string  $query
     * @param  int  $dialect
     * @return Relay|string|false
     */
    #[Attributes\RedisCommand]
    public function ftExplain(mixed $index, string $query, int $dialect = 0): Relay|string|false;

    /**
     * Return the execution plan for a complex query but formatted for easier reading from CLI.
     *
     * @param  mixed  $index
     * @param  string  $query
     * @param  int  $dialect
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftExplainCli(mixed $index, string $query, int $dialect = 0): Relay|array|false;

    /**
     * Returns information and statistics about a given index.
     *
     * @param  mixed  $index
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftInfo(mixed $index): Relay|array|false;

    /**
     * Apply FT.SEARCH or FT.AGGREGATE command to collect performance details.
     *
     * @param  mixed  $index
     * @param  string  $command
     * @param  string  $query
     * @param  bool  $limited
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftProfile(mixed $index, string $command, string $query, bool $limited = false): Relay|array|false;

    /**
     * Search the index with a textual query, returning either documents or just ids.
     *
     * @param  mixed  $index
     * @param  string  $query
     * @param  array|null  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftSearch(mixed $index, string $query, ?array $options = null): Relay|array|false;

    /**
     * Perform spelling correction on a query, returning suggestions for misspelled terms.
     *
     * @param  mixed  $index
     * @param  string  $query
     * @param  array|null  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftSpellCheck(mixed $index, string $query, ?array $options = null): Relay|array|false;

    /**
     * Dump the contents of a synonym group.
     *
     * @param  mixed  $index
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftSynDump(mixed $index): Relay|array|false;

    /**
     * Update a synonym group.
     *
     * @param  mixed  $index
     * @param  string  $synonym
     * @param  mixed  $term_or_terms
     * @param  bool  $skipinitialscan
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ftSynUpdate(mixed $index, string $synonym, mixed $term_or_terms, bool $skipinitialscan = false): Relay|bool;

    /**
     * Return a distinct set of values indexed in a Tag field.
     *
     * @param  mixed  $index
     * @param  string  $tag
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function ftTagVals(mixed $index, string $tag): Relay|array|false;

    /**
     * Returns the number of keys in the currently-selected database.
     *
     * @return Relay|int
     */
    #[Attributes\RedisCommand]
    public function dbsize(): Relay|int|false;

    /**
     * Serialize and return the value stored at key in a Redis-specific format.
     *
     * @param  mixed  $key
     * @return Relay|string|null|false
     */
    #[Attributes\RedisCommand]
    public function dump(mixed $key): Relay|string|null|false;

    /**
     * Attach or detach the instance as a replica of another instance.
     *
     * @param  string|null  $host
     * @param  int  $port
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function replicaof(?string $host = null, $port = 0): Relay|bool;

    /**
     * Pause the client until sufficient local and/or remote AOF data has been flushed to disk.
     *
     * @param  int  $numlocal
     * @param  int  $numremote
     * @return Relay|array
     */
    #[Attributes\RedisCommand]
    public function waitaof(int $numlocal, int $numremote, int $timeout): Relay|array|false;

    /**
     * Create a key associated with a value that is obtained by deserializing the provided serialized value.
     *
     * @param  mixed  $key
     * @param  int  $ttl
     * @param  string  $value
     * @param  array  $options
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function restore(mixed $key, int $ttl, string $value, ?array $options = null): Relay|bool;

    /**
     * Atomically transfer a key from a Redis instance to another one.
     *
     * @param  string  $host
     * @param  int  $port
     * @param  string|array  $key
     * @param  int  $dstdb
     * @param  int  $timeout
     * @param  bool  $copy
     * @param  bool  $replace
     * @param  mixed  $credentials
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function migrate(
        string $host,
        int $port,
        string|array $key,
        int $dstdb,
        int $timeout,
        bool $copy = false,
        bool $replace = false,
        #[\SensitiveParameter]
        mixed $credentials = null
    ): Relay|bool;

    /**
     * This command copies the value stored at the source key to the destination key.
     *
     * @param  mixed  $src
     * @param  mixed  $dst
     * @param  array  $options
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function copy(mixed $src, mixed $dst, ?array $options = null): Relay|bool;

    /**
     * Asks Redis to echo back the provided string.
     *
     * @param  string  $arg
     * @return Relay|bool|string
     */
    #[Attributes\RedisCommand]
    public function echo(string $arg): Relay|bool|string;

    /**
     * Returns PONG if no argument is provided, otherwise return a copy of the argument as a bulk.
     *
     * @param  string  $arg
     * @return Relay|bool|string
     */
    #[Attributes\RedisCommand]
    public function ping(string $arg = null): Relay|bool|string;

    /**
     * Returns the number of milliseoconds since Relay has seen activity from the server.
     *
     * @return Relay|int|false
     */
    #[Attributes\Local]
    public function idleTime(): Relay|int|false;

    /**
     * Returns a random key from Redis.
     *
     * @return Relay|string|null|bool
     */
    #[Attributes\RedisCommand]
    public function randomkey(): Relay|string|null|bool;

    /**
     * Returns the current time from Redis.
     *
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function time(): Relay|array|false;

    /**
     * Asynchronously rewrite the append-only file.
     *
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function bgrewriteaof(): Relay|bool;

    /**
     * Returns the UNIX time stamp of the last successful save to disk.
     *
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function lastsave(): Relay|int|false;

    /**
     * Get the longest common subsequence between two string keys.
     *
     * @param  mixed  $key1
     * @param  mixed  $key2
     * @param  array|null  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function lcs(mixed $key1, mixed $key2, ?array $options = null): mixed;

    /**
     * Asynchronously save the dataset to disk.
     *
     * @param  bool  $schedule
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function bgsave(bool $schedule = false): Relay|bool;

    /**
     * Synchronously save the dataset to disk.
     *
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function save(): Relay|bool;

    /**
     * Returns the role of the instance in the context of replication.
     *
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function role(): Relay|array|false;

    /**
     * Returns the remaining time to live of a key that has a timeout in seconds.
     *
     * @param  mixed  $key
     * @return Relay|int
     */
    #[Attributes\RedisCommand]
    public function ttl(mixed $key): Relay|int|false;

    /**
     * Returns the remaining time to live of a key that has a timeout in milliseconds.
     *
     * @param  mixed  $key
     * @return Relay|int
     */
    #[Attributes\RedisCommand]
    public function pttl(mixed $key): Relay|int|false;

    /**
     * Returns if key(s) exists.
     *
     * @param  mixed  $keys,...
     * @return Relay|bool|int
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function exists(mixed ...$keys): Relay|bool|int;

    /**
     * Evaluate script using the Lua interpreter.
     *
     * @see https://redis.io/commands/eval
     *
     * @param  mixed  $script
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function eval(mixed $script, array $args = [], int $num_keys = 0): mixed;

    /**
     * Evaluate script using the Lua interpreter.  This is just the "read-only" variant of EVAL
     * meaning it can be run on read-only replicas.
     *
     * @see https://redis.io/commands/eval_ro
     *
     * @param  mixed  $script
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function eval_ro(mixed $script, array $args = [], int $num_keys = 0): mixed;

    /**
     * Evaluates a script cached on the server-side by its SHA1 digest.
     *
     *
     * @param  string  $sha
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function evalsha(string $sha, array $args = [], int $num_keys = 0): mixed;

    /**
     * Evaluates a script cached on the server-side by its SHA1 digest.  This is just the "read-only" variant
     * of `EVALSHA` meaning it can be run on read-only replicas.
     *
     * @param  string  $sha
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function evalsha_ro(string $sha, array $args = [], int $num_keys = 0): mixed;

    /**
     * Executes `CLIENT` command operations.
     *
     * @param  string  $operation
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function client(string $operation, mixed ...$args): mixed;

    /**
     * Add one or more members to a geospacial sorted set
     *
     * @param  string  $key
     * @param  float  $lng
     * @param  float  $lat
     * @param  string  $member
     * @param  mixed  $other_triples_and_options,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function geoadd(
        string $key,
        float $lng,
        float $lat,
        string $member,
        mixed ...$other_triples_and_options
    ): Relay|int|false;

    /**
     * Get the distance between two members of a geospacially encoded sorted set.
     *
     * @param  string  $key
     * @param  string  $src
     * @param  string  $dst
     * @param  string|null  $unit
     * @return Relay|float|null|false
     */
    #[Attributes\RedisCommand]
    public function geodist(string $key, string $src, string $dst, ?string $unit = null): Relay|float|null|false;

    /**
     * Retrieve one or more GeoHash encoded strings for members of the set.
     *
     * @param  string  $key
     * @param  string  $member
     * @param  string  $other_members,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function geohash(string $key, string $member, string ...$other_members): Relay|array|false;

    /**
     * Retrieve members of a geospacially sorted set that are within a certain radius of a location.
     *
     * @param  string  $key
     * @param  float  $lng
     * @param  float  $lat
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadius(string $key, float $lng, float $lat, float $radius, string $unit, array $options = []): mixed;

    /**
     * Similar to `GEORADIUS` except it uses a member as the center of the query.
     *
     * @param  string  $key
     * @param  string  $member
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadiusbymember(string $key, string $member, float $radius, string $unit, array $options = []): mixed;

    /**
     * Similar to `GEORADIUS` except it uses a member as the center of the query.
     *
     * @param  string  $key
     * @param  string  $member
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadiusbymember_ro(string $key, string $member, float $radius, string $unit, array $options = []): mixed;

    /**
     * Retrieve members of a geospacially sorted set that are within a certain radius of a location.
     *
     * @param  string  $key
     * @param  float  $lng
     * @param  float  $lat
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadius_ro(string $key, float $lng, float $lat, float $radius, string $unit, array $options = []): mixed;

    /**
     * Search a geospacial sorted set for members in various ways.
     *
     * @param  string  $key
     * @param  array|string  $position
     * @param  array|int|float  $shape
     * @param  string  $unit
     * @param  array  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function geosearch(
        string $key,
        array|string $position,
        array|int|float $shape,
        string $unit,
        array $options = []
    ): Relay|array|false;

    /**
     * Search a geospacial sorted set for members within a given area or range, storing the results into
     * a new set.
     *
     * @param  string  $dst
     * @param  string  $src
     * @param  array|string  $position
     * @param  array|int|float  $shape
     * @param  string  $unit
     * @param  array  $options
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function geosearchstore(
        string $dst,
        string $src,
        array|string $position,
        array|int|float $shape,
        string $unit,
        array $options = []
    ): Relay|int|false;

    /**
     * Get the value of key.
     *
     * @param  mixed  $key
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function get(mixed $key): mixed;

    /**
     * Atomically sets key to value and returns the old value stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function getset(mixed $key, mixed $value): mixed;

    /**
     * Returns the substring of the string value stored at key,
     * determined by the offsets start and end (both are inclusive).
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function getrange(mixed $key, int $start, int $end): mixed;

    /**
     * Overwrites part of the string stored at key, starting at
     * the specified offset, for the entire length of value.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  mixed  $value
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function setrange(mixed $key, int $start, mixed $value): Relay|int|false;

    /**
     * Returns the bit value at offset in the string value stored at key.
     *
     * @param  mixed  $key
     * @param  int  $pos
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function getbit(mixed $key, int $pos): Relay|int|false;

    /**
     * Count the number of set bits (population counting) in a string.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @param  bool  $by_bit
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function bitcount(mixed $key, int $start = 0, int $end = -1, bool $by_bit = false): Relay|int|false;

    /**
     * Perform various bitfield operations on a string key, such as getting/setting bit ranges,
     * incrementing, etc.
     *
     * @param  mixed  $key
     * @param  mixed  $args,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function bitfield(mixed $key, mixed ...$args): Relay|array|false;

    /**
     * This is a container command for runtime configuration commands.
     *
     * @param  string  $operation
     * @param  mixed  $key
     * @param  string|null  $value
     * @return Relay|array|bool
     */
    #[Attributes\RedisCommand]
    public function config(string $operation, mixed $key = null, ?string $value = null): Relay|array|bool;

    /**
     * Return an array with details about every Redis command.
     *
     * @param  array  $args,...
     * @return Relay|array|int|false
     */
    #[Attributes\RedisCommand]
    public function command(mixed ...$args): Relay|array|int|false;

    /**
     * Perform a bitwise operation on one or more keys, storing the result in a new key.
     *
     * @param  string  $operation
     * @param  string  $dstkey
     * @param  string  $srckey
     * @param  string  $other_keys,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function bitop(string $operation, string $dstkey, string $srckey, string ...$other_keys): Relay|int|false;

    /**
     * Return the position of the first bit set to 1 or 0 in a string.
     *
     * @param  mixed  $key
     * @param  int  $bit
     * @param  int  $start
     * @param  int  $end
     * @param  bool  $bybit
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function bitpos(mixed $key, int $bit, int $start = null, int $end = null, bool $bybit = false): Relay|int|false;

    /**
     * Sets or clears the bit at offset in the string value stored at key.
     *
     * @param  mixed  $key
     * @param  int  $pos
     * @param  int  $val
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function setbit(mixed $key, int $pos, int $val): Relay|int|false;

    /**
     * Interact with Redis' ACLs
     *
     * @param  string  $cmd
     * @param  string  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function acl(string $cmd, string ...$args): mixed;

    /**
     * If key already exists and is a string, this command appends
     * the value at the end of the string. If key does not exist
     * it is created and set as an empty string, so APPEND will
     * be similar to SET in this special case.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function append(mixed $key, mixed $value): Relay|int|false;

    /**
     * Set key to hold the string value. If key already holds
     * a value, it is overwritten, regardless of its type.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @param  mixed  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function set(mixed $key, mixed $value, mixed $options = null): mixed;

    /**
     * Get the value of key and optionally set its expiration.
     * GETEX is similar to GET, but is a write command with additional options.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function getex(mixed $key, ?array $options = null): mixed;

    /**
     * Get the value of key and delete the key. This command is similar to GET,
     * except for the fact that it also deletes the key on success
     * (if and only if the key's value type is a string).
     *
     * @param  mixed  $key
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function getdel(mixed $key): mixed;

    /**
     * Set key to hold the string value and set key to timeout after a given number of seconds.
     *
     * @param  mixed  $key
     * @param  int  $seconds
     * @param  mixed  $value
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function setex(mixed $key, int $seconds, mixed $value): Relay|bool;

    /**
     * Adds the specified elements to the specified HyperLogLog.
     *
     * @param  string  $key
     * @param  array  $elements
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function pfadd(string $key, array $elements): Relay|int|false;

    /**
     * Return the approximated cardinality of the set(s) observed by the HyperLogLog at key(s).
     *
     * @param  string|array  $key_or_keys
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function pfcount(string|array $key_or_keys): Relay|int|false;

    /**
     * Merge given HyperLogLogs into a single one.
     *
     * @param  string  $dst
     * @param  array  $srckeys
     * @return Relay|bool
     */
    public function pfmerge(string $dst, array $srckeys): Relay|bool;

    /**
     * Set key to hold the string value and set key to timeout after a given number of milliseconds.
     *
     *
     * @param  mixed  $key
     * @param  int  $milliseconds
     * @param  mixed  $value
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function psetex(mixed $key, int $milliseconds, mixed $value): Relay|bool;

    /**
     * Posts a message to the given channel.
     *
     * @param  string  $channel
     * @param  string  $message
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function publish(string $channel, string $message): Relay|int|false;

    /**
     * A container command for Pub/Sub introspection commands.
     *
     * @param  string  $operation
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function pubsub(string $operation, mixed ...$args): mixed;

    /**
     * Posts a message to the given shard channel.
     *
     * @param  string  $channel
     * @param  string  $message
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function spublish(string $channel, string $message): Relay|int|false;

    /**
     * Set key to hold string value if key does not exist. In that case, it is equal to SET.
     * When key already holds a value, no operation is performed.
     * SETNX is short for "SET if Not eXists".
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function setnx(mixed $key, mixed $value): Relay|bool;

    /**
     * Returns the values of all specified keys.
     *
     * @param  array  $keys
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function mget(array $keys): Relay|array|false;

    /**
     * Move key from the currently selected database to the specified destination database.
     *
     * @param  mixed  $key
     * @param  int  $db
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function move(mixed $key, int $db): Relay|int|false;

    /**
     * Sets the given keys to their respective values.
     * MSET replaces existing values with new values, just as regular SET.
     *
     * @param  array  $kvals
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function mset(array $kvals): Relay|bool;

    /**
     * Sets the given keys to their respective values.
     * MSETNX will not perform any operation at all even if just a single key already exists.
     *
     * @param  array  $kvals
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function msetnx(array $kvals): Relay|bool;

    /**
     * Renames key.
     *
     * @param  mixed  $key
     * @param  mixed  $newkey
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function rename(mixed $key, mixed $newkey): Relay|bool;

    /**
     * Renames key if the new key does not yet exist.
     *
     * @param  mixed  $key
     * @param  mixed  $newkey
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function renamenx(mixed $key, mixed $newkey): Relay|bool;

    /**
     * Removes the specified keys.
     *
     * @param  mixed  $keys,...
     * @return Relay|int|bool
     */
    #[Attributes\RedisCommand]
    public function del(mixed ...$keys): Relay|int|bool;

    /**
     * Removes the specified keys without blocking Redis.
     *
     * @param  mixed  $keys,...
     * @return Relay|int
     */
    #[Attributes\RedisCommand]
    public function unlink(mixed ...$keys): Relay|int|false;

    /**
     * Set a timeout on key.
     *
     * @param  mixed  $key
     * @param  int  $seconds
     * @param  string|null  $mode
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function expire(mixed $key, int $seconds, ?string $mode = null): Relay|bool;

    /**
     * Set a key's time to live in milliseconds.
     *
     * @param  mixed  $key
     * @param  int  $milliseconds
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function pexpire(mixed $key, int $milliseconds): Relay|bool;

    /**
     * Set a timeout on key using a unix timestamp.
     *
     * @param  mixed  $key
     * @param  int  $timestamp
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function expireat(mixed $key, int $timestamp): Relay|bool;

    /**
     * Returns the absolute Unix timestamp in seconds at which the given key will expire.
     * If the key exists but doesn't have a TTL this function return -1.
     * If the key does not exist -2.
     *
     * @param  mixed  $key
     * @return Relay|int|false
     * */
    #[Attributes\RedisCommand]
    public function expiretime(mixed $key): Relay|int|false;

    /**
     * Set the expiration for a key as a UNIX timestamp specified in milliseconds.
     *
     * @param  mixed  $key
     * @param  int  $timestamp_ms
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function pexpireat(mixed $key, int $timestamp_ms): Relay|bool;

    /**
     * Semantic the same as EXPIRETIME, but returns the absolute Unix expiration
     * timestamp in milliseconds instead of seconds.
     *
     * @param  mixed  $key
     * @return Relay|int|false
     * */
    #[Attributes\RedisCommand]
    public function pexpiretime(mixed $key): Relay|int|false;

    /**
     * Remove the existing timeout on key, turning the key from volatile to persistent.
     *
     * @param  mixed  $key
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function persist(mixed $key): Relay|bool;

    /**
     * Returns the type of a given key.
     *
     * In PhpRedis compatibility mode this will return an integer
     * (one of the REDIS_<type>) constants. Otherwise it will
     * return the string that Redis returns.
     *
     * @param  mixed  $key
     * @return Relay|int|string|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function type(mixed $key): Relay|int|string|bool;

    /**
     * Atomically returns and removes the first/last element of the list
     * stored at source, and pushes the element at the first/last
     * element of the list stored at destination.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @param  string  $srcpos
     * @param  string  $dstpos
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function lmove(mixed $srckey, mixed $dstkey, string $srcpos, string $dstpos): mixed;

    /**
     * BLMOVE is the blocking variant of LMOVE. When source contains elements,
     * this command behaves exactly like LMOVE. When used inside a
     * MULTI/EXEC block, this command behaves exactly like LMOVE.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @param  string  $srcpos
     * @param  string  $dstpos
     * @param  float  $timeout
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function blmove(mixed $srckey, mixed $dstkey, string $srcpos, string $dstpos, float $timeout): mixed;

    /**
     * Returns the specified elements of the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $stop
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function lrange(mixed $key, int $start, int $stop): Relay|array|false;

    /**
     * Insert all the specified values at the head of the list stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  mixed  $mems,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function lpush(mixed $key, mixed $mem, mixed ...$mems): Relay|int|false;

    /**
     * Insert all the specified values at the tail of the list stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  mixed  $mems,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function rpush(mixed $key, mixed $mem, mixed ...$mems): Relay|int|false;

    /**
     * Inserts specified values at the head of the list stored at key,
     * only if key already exists and holds a list.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  mixed  $mems,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function lpushx(mixed $key, mixed $mem, mixed ...$mems): Relay|int|false;

    /**
     * Inserts specified values at the tail of the list stored at key,
     * only if key already exists and holds a list.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  mixed  $mems,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function rpushx(mixed $key, mixed $mem, mixed ...$mems): Relay|int|false;

    /**
     * Sets the list element at index to element.
     *
     * @param  mixed  $key
     * @param  int  $index
     * @param  mixed  $mem
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function lset(mixed $key, int $index, mixed $mem): Relay|bool;

    /**
     * Removes and returns the first elements of the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function lpop(mixed $key, int $count = 1): mixed;

    /**
     * The command returns the index of matching elements inside a Redis list.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @param  array  $options
     * @return Relay|int|array|false|null
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function lpos(mixed $key, mixed $value, ?array $options = null): Relay|int|array|false|null;

    /**
     * Removes and returns the last elements of the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function rpop(mixed $key, int $count = 1): mixed;

    /**
     * Atomically returns and removes the last element (tail) of the list stored at source,
     * and pushes the element at the first element (head) of the list stored at destination.
     *
     * @param  mixed  $source
     * @param  mixed  $dest
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function rpoplpush(mixed $source, mixed $dest): mixed;

    /**
     * Atomically returns and removes the last element (tail) of the list stored at source,
     * and pushes the element at the first element (head) of the list stored at destination.
     * This command will block for an element up to the provided timeout.
     *
     * @param  mixed  $source
     * @param  mixed  $dest
     * @param  float  $timeout
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function brpoplpush(mixed $source, mixed $dest, float $timeout): mixed;

    /**
     * BLPOP is a blocking list pop primitive. It is the blocking version of LPOP because
     * it blocks the connection when there are no elements to pop from any of the given lists.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function blpop(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Relay|array|null|false;

    /**
     * Pop elements from a list, or block until one is available
     *
     * @param  float  $timeout
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function blmpop(float $timeout, array $keys, string $from, int $count = 1): Relay|array|null|false;

    /**
     * Remove and return members with scores in a sorted set or block until one is available
     *
     * @param  float  $timeout
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function bzmpop(float $timeout, array $keys, string $from, int $count = 1): Relay|array|null|false;

    /**
     * Pops one or more elements from the first non-empty list key from the list of provided key names.
     *
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function lmpop(array $keys, string $from, int $count = 1): Relay|array|null|false;

    /**
     * Pops one or more elements, that are member-score pairs, from the
     * first non-empty sorted set in the provided list of key names.
     *
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function zmpop(array $keys, string $from, int $count = 1): Relay|array|null|false;

    /**
     * BRPOP is a blocking list pop primitive. It is the blocking version of RPOP because
     * it blocks the connection when there are no elements to pop from any of the given lists.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function brpop(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Relay|array|null|false;

    /**
     * BZPOPMAX is the blocking variant of the sorted set ZPOPMAX primitive.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function bzpopmax(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Relay|array|null|false;

    /**
     * BZPOPMIN is the blocking variant of the sorted set ZPOPMIN primitive.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Relay|array|null|false
     */
    #[Attributes\RedisCommand]
    public function bzpopmin(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Relay|array|null|false;

    /**
     * This is a container command for object introspection commands.
     *
     * @param  string  $op
     * @param  mixed  $key
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function object(string $op, mixed $key): mixed;

    /**
     * Return the positions (longitude,latitude) of all the specified members
     * of the geospatial index represented by the sorted set at key.
     *
     * @param  mixed  $key
     * @param  mixed  $members,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function geopos(mixed $key, mixed ...$members): Relay|array|false;

    /**
     * Removes the first count occurrences of elements equal to element from the list stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  int  $count
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function lrem(mixed $key, mixed $mem, int $count = 0): Relay|int|false;

    /**
     * Returns the element at index index in the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $index
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function lindex(mixed $key, int $index): mixed;

    /**
     * Inserts element in the list stored at key either before or after the reference value pivot.
     *
     * @param  mixed  $key
     * @param  string  $op
     * @param  mixed  $pivot
     * @param  mixed  $element
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function linsert(mixed $key, string $op, mixed $pivot, mixed $element): Relay|int|false;

    /**
     * Trim an existing list so that it will contain only the specified range of elements specified.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function ltrim(mixed $key, int $start, int $end): Relay|bool;

    /**
     * Returns the value associated with field in the hash stored at key.
     *
     * @param  mixed  $hash
     * @param  mixed  $member
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hget(mixed $hash, mixed $member): mixed;

    /**
     * Returns the string length of the value associated with field in the hash stored at key.
     *
     * @param  mixed  $hash
     * @param  mixed  $member
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hstrlen(mixed $hash, mixed $member): Relay|int|false;

    /**
     * Returns all fields and values of the hash stored at key.
     *
     * @param  mixed  $hash
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hgetall(mixed $hash): Relay|array|false;

    /**
     * Returns all field names in the hash stored at key.
     *
     * @param  mixed  $hash
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hkeys(mixed $hash): Relay|array|false;

    /**
     * Returns all values in the hash stored at key.
     *
     * @param  mixed  $hash
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hvals(mixed $hash): Relay|array|false;

    /**
     * Returns the values associated with the specified fields in the hash stored at key.
     *
     * @param  mixed  $hash
     * @param  array  $members
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hmget(mixed $hash, array $members): Relay|array|false;

    /**
     * When called with just the key argument, return a random field from the hash value stored at key.
     *
     * @param  mixed  $hash
     * @param  array  $options
     * @return Relay|array|string|null|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hrandfield(mixed $hash, ?array $options = null): Relay|array|string|null|false;

    /**
     * Sets the specified fields to their respective values in the hash stored at key.
     *
     * @param  mixed  $hash
     * @param  array  $members
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function hmset(mixed $hash, array $members): Relay|bool;

    /**
     * Returns if field is an existing field in the hash stored at key.
     *
     * @param  mixed  $hash
     * @param  mixed  $member
     * @return Relay|bool
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hexists(mixed $hash, mixed $member): Relay|bool;

    /**
     * Sets field in the hash stored at key to value, only if field does not yet exist.
     *
     * @param  mixed  $hash
     * @param  mixed  $member
     * @param  mixed  $value
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function hsetnx(mixed $hash, mixed $member, mixed $value): Relay|bool;

    /**
     * Sets field in the hash stored at key to value.
     *
     * @param  mixed  $key
     * @param  mixed  $keys_and_vals...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function hset(mixed $key, mixed ...$keys_and_vals): Relay|int|false;

    /**
     * Removes the specified fields from the hash stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  string  $mems,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function hdel(mixed $key, mixed $mem, string ...$mems): Relay|int|false;

    /**
     * Increments the number stored at field in the hash stored at key by increment.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  int  $value
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function hincrby(mixed $key, mixed $mem, int $value): Relay|int|false;

    /**
     * Increment the specified field of a hash stored at key, and representing
     * a floating point number, by the specified increment.
     *
     * @param  mixed  $key
     * @param  mixed  $mem
     * @param  float  $value
     * @return Relay|float|bool
     */
    #[Attributes\RedisCommand]
    public function hincrbyfloat(mixed $key, mixed $mem, float $value): Relay|float|bool;

    /**
     * Increments the number stored at key by one.
     *
     * @param  mixed  $key
     * @param  int  $by
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function incr(mixed $key, int $by = 1): Relay|int|false;

    /**
     * Decrements the number stored at key by one.
     *
     * @param  mixed  $key
     * @param  int  $by
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function decr(mixed $key, int $by = 1): Relay|int|false;

    /**
     * Increments the number stored at key by increment.
     *
     * @param  mixed  $key
     * @param  int  $value
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function incrby(mixed $key, int $value): Relay|int|false;

    /**
     * Decrements the number stored at key by decrement.
     *
     * @param  mixed  $key
     * @param  int  $value
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function decrby(mixed $key, int $value): Relay|int|false;

    /**
     * Increment the string representing a floating point number stored at key by the specified increment.
     *
     * @param  mixed  $key
     * @param  float  $value
     * @return Relay|float|false
     */
    #[Attributes\RedisCommand]
    public function incrbyfloat(mixed $key, float $value): Relay|float|false;

    /**
     * Append the json values into the array at path after the last element in it.
     *
     * @param  mixed  $key
     * @param  mixed  $value_or_array
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonArrAppend(mixed $key, mixed $value_or_array, ?string $path = null): Relay|array|false;

    /**
     * Search for the first occurrence of a JSON value in an array.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  mixed  $value
     * @param  int|null  $start
     * @param  int|null  $stop
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonArrIndex(mixed $key, string $path, mixed $value, ?int $start = 0, ?int $stop = -1): Relay|array|false;

    /**
     * Insert the json values into the array at path before the index (shifts to the right).
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  int  $index
     * @param  mixed  $value
     * @param  mixed  $other_values,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonArrInsert(mixed $key, string $path, int $index, mixed $value, mixed ...$other_values): Relay|array|false;

    /**
     * Report the length of the JSON array at path in key.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonArrLen(mixed $key, ?string $path = null): Relay|array|false;

    /**
     * Remove and return an element from the index in the array.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @param  int  $index
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonArrPop(mixed $key, ?string $path = null, int $index = -1): Relay|array|false;

    /**
     * Trim an array so that it contains only the specified inclusive range of elements.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  int  $start
     * @param  int  $stop
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonArrTrim(mixed $key, string $path, int $start, int $stop): Relay|array|false;

    /**
     * Clear container values (arrays/objects) and set numeric values to 0.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function jsonClear(mixed $key, ?string $path = null): Relay|int|false;

    /**
     * Container command for JSON debugging related tasks.
     *
     * @param  string  $command
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function jsonDebug(string $command, mixed $key, ?string $path = null): Relay|int|false;

    /**
     * Delete a value.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function jsonDel(mixed $key, ?string $path = null): Relay|int|false;

    /**
     * @see Relay::jsonDel
     */
    #[Attributes\RedisCommand]
    public function jsonForget(mixed $key, ?string $path = null): Relay|int|false;

    /**
     * Return the value at path in JSON serialized form.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @param  string  $paths,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function jsonGet(mixed $key, array $options = [], string ...$paths): mixed;

    /**
     * Merge a given JSON value into matching paths. Consequently, JSON values
     * at matching paths are updated, deleted, or expanded with new children.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  mixed  $value
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function jsonMerge(mixed $key, string $path, mixed $value): Relay|bool;

    /**
     * Return the values at path from multiple key arguments.
     *
     * @param  mixed  $key_or_array
     * @param  string  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonMget(mixed $key_or_array, string $path): Relay|array|false;

    /**
     * Set or update one or more JSON values according to the specified key-path-value triplets.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  mixed  $value
     * @param  mixed  $other_triples
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function jsonMset(mixed $key, string $path, mixed $value, mixed ...$other_triples): Relay|bool;

    /**
     * Increment the number value stored at path by number.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  int  $value
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonNumIncrBy(mixed $key, string $path, int $value): Relay|array|false;

    /**
     * Multiply the number value stored at path by number.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  int  $value
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonNumMultBy(mixed $key, string $path, int $value): Relay|array|false;

    /**
     * Return the keys in the object that's referenced by path.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonObjKeys(mixed $key, ?string $path = null): Relay|array|false;

    /**
     * Report the number of keys in the JSON object at path in key.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonObjLen(mixed $key, ?string $path = null): Relay|array|false;

    /**
     * Return the JSON in key in RESP specification form.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|array|string|int|false
     */
    #[Attributes\RedisCommand]
    public function jsonResp(mixed $key, ?string $path = null): Relay|array|string|int|false;

    /**
     * Set the JSON value at path in key.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @param  mixed  $value
     * @param  string|null  $condition
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function jsonSet(mixed $key, string $path, mixed $value, ?string $condition = null): Relay|bool;

    /**
     * Append the json-string values to the string at path.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @param  string|null  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonStrAppend(mixed $key, mixed $value, ?string $path = null): Relay|array|false;

    /**
     * Report the length of the JSON String at path in key.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonStrLen(mixed $key, ?string $path = null): Relay|array|false;

    /**
     * Toggle a Boolean value stored at path.
     *
     * @param  mixed  $key
     * @param  string  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonToggle(mixed $key, string $path): Relay|array|false;

    /**
     * Report the type of JSON value at path.
     *
     * @param  mixed  $key
     * @param  string|null  $path
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function jsonType(mixed $key, ?string $path = null): Relay|array|false;

    /**
     * Returns the members of the set resulting from the difference
     * between the first set and all the successive sets.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sdiff(mixed $key, mixed ...$other_keys): Relay|array|false;

    /**
     * This command is equal to SDIFF, but instead of returning the resulting set,
     * it is stored in destination. If destination already exists, it is overwritten.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function sdiffstore(mixed $key, mixed ...$other_keys): Relay|int|false;

    /**
     * Returns the members of the set resulting from the intersection of all the given sets.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sinter(mixed $key, mixed ...$other_keys): Relay|array|false;

    /**
     * Intersect multiple sets and return the cardinality of the result.
     *
     * @param  array  $keys
     * @param  int  $limit
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sintercard(array $keys, int $limit = -1): Relay|int|false;

    /**
     * This command is equal to SINTER, but instead of returning the resulting set,
     * it is stored in destination. If destination already exists, it is overwritten.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function sinterstore(mixed $key, mixed ...$other_keys): Relay|int|false;

    /**
     * Returns the members of the set resulting from the union of all the given sets.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sunion(mixed $key, mixed ...$other_keys): Relay|array|false;

    /**
     * This command is equal to SUNION, but instead of returning the resulting set,
     * it is stored in destination. If destination already exists, it is overwritten.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function sunionstore(mixed $key, mixed ...$other_keys): Relay|int|false;

    /**
     * Subscribes to the specified channels.
     *
     * @param  array  $channels
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function subscribe(array $channels, callable $callback): bool;

    /**
     * Unsubscribes from the given channels, or from all of them if none is given.
     *
     * @param  array  $channels
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function unsubscribe(array $channels = []): bool;

    /**
     * Subscribes to the given patterns.
     *
     * @param  array  $patterns
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function psubscribe(array $patterns, callable $callback): bool;

    /**
     * Unsubscribes from the given patterns, or from all of them if none is given.
     *
     * @param  array  $patterns
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function punsubscribe(array $patterns = []): bool;

    /**
     * Subscribes to the specified shard channels.
     *
     * @param  array  $channels
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function ssubscribe(array $channels, callable $callback): bool;

    /**
     * Unsubscribes from the given shard channels, or from all of them if none is given.
     *
     * @param  array  $channels
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function sunsubscribe(array $channels = []): bool;

    /**
     * Alters the last access time of a key(s).
     *
     * @param  array|string  $key_or_array
     * @param  mixed  $more_keys,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function touch(array|string $key_or_array, mixed ...$more_keys): Relay|int|false;

    /**
     * A pipeline block is simply transmitted faster to the server (like `MULTI`), but without any guarantee of atomicity.
     *
     * @return Relay|bool
     */
    #[Attributes\Local]
    public function pipeline(): Relay|bool;

    /**
     * Marks the start of a transaction block. Subsequent commands will be queued for atomic execution using EXEC.
     *
     * Accepts `Relay::MULTI` and `Relay::PIPELINE` modes.
     *
     * @param  int  $mode
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function multi(int $mode = 0): Relay|bool;

    /**
     * Executes all previously queued commands in a transaction and restores the connection state to normal.
     *
     * @return Relay|array|bool
     */
    #[Attributes\RedisCommand]
    public function exec(): Relay|array|bool;

    /**
     * Wait for the synchronous replication of all the write
     * commands sent in the context of the current connection.
     *
     * @param  int  $replicas
     * @param  int  $timeout
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function wait(int $replicas, $timeout): Relay|int|false;

    /**
     * Marks the given keys to be watched for conditional execution of a transaction.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function watch(mixed $key, mixed ...$other_keys): Relay|bool;

    /**
     * Flushes all the previously watched keys for a transaction.
     * If you call EXEC or DISCARD, there's no need to manually call UNWATCH.
     *
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function unwatch(): Relay|bool;

    /**
     * Flushes all previously queued commands in a transaction and restores the connection state to normal.
     * If WATCH was used, DISCARD unwatches all keys watched by the connection.
     *
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function discard(): bool;

    /**
     * Get the mode Relay is currently in.
     * `Relay::ATOMIC`, `Relay::PIPELINE` or `Relay::MULTI`.
     *
     * @param  bool  $masked
     * @return int
     */
    #[Attributes\Local]
    public function getMode(bool $masked = false): int;

    /**
     * Clear the accumulated sent and received bytes.
     *
     * @return void
     */
    #[Attributes\Local]
    public function clearBytes(): void;

    /**
     * Scan the keyspace for matching keys.
     *
     * @param  mixed  $iterator
     * @param  mixed  $match
     * @param  int  $count
     * @param  string|null  $type
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function scan(mixed &$iterator, mixed $match = null, int $count = 0, ?string $type = null): array|false;

    /**
     * Iterates fields of Hash types and their associated values.
     *
     * @param  mixed  $key
     * @param  mixed  $iterator
     * @param  mixed  $match
     * @param  int  $count
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function hscan(mixed $key, mixed &$iterator, mixed $match = null, int $count = 0): array|false;

    /**
     * Iterates elements of Sets types.
     *
     * @param  mixed  $key
     * @param  mixed  $iterator
     * @param  mixed  $match
     * @param  int  $count
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function sscan(mixed $key, mixed &$iterator, mixed $match = null, int $count = 0): array|false;

    /**
     * Iterates elements of Sorted Set types and their associated scores.
     *
     * @param  mixed  $key
     * @param  mixed  $iterator
     * @param  mixed  $match
     * @param  int  $count
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function zscan(mixed $key, mixed &$iterator, mixed $match = null, int $count = 0): array|false;

    /**
     * Returns all keys matching pattern.
     *
     * @param  mixed  $pattern
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function keys(mixed $pattern): Relay|array|false;

    /**
     * Interact with the Redis slowlog.
     *
     * @param  string  $operation
     * @param  string  $extra_args,...
     * @return Relay|array|int|bool
     */
    #[Attributes\RedisCommand]
    public function slowlog(string $operation, string ...$extra_args): Relay|array|int|bool;

    /**
     * Returns all the members of the set value stored at `$key`.
     *
     * @param  mixed  $set
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function smembers(mixed $set): Relay|array|false;

    /**
     * Returns if `$member` is a member of the set stored at `$key`.
     *
     * @param  mixed  $set
     * @param  mixed  $member
     * @return Relay|bool
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sismember(mixed $set, mixed $member): Relay|bool;

    /**
     * Returns whether each member is a member of the set stored at `$key`.
     *
     * @param  mixed  $set
     * @param  mixed  $members,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function smismember(mixed $set, mixed ...$members): Relay|array|false;

    /**
     * Remove the specified members from the set stored at `$key`.
     *
     * @param  mixed  $set
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function srem(mixed $set, mixed $member, mixed ...$members): Relay|int|false;

    /**
     * Add the specified members to the set stored at `$key`.
     *
     * @param  mixed  $set
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function sadd(mixed $set, mixed $member, mixed ...$members): Relay|int|false;

    /**
     * Sort the elements in a list, set or sorted set.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @return Relay|array|int|false
     */
    #[Attributes\RedisCommand]
    public function sort(mixed $key, array $options = []): Relay|array|int|false;

    /**
     * Sort the elements in a list, set or sorted set. Read-only variant of SORT.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function sort_ro(mixed $key, array $options = []): Relay|array|false;

    /**
     * Move member from the set at source to the set at destination.
     *
     * @param  mixed  $srcset
     * @param  mixed  $dstset
     * @param  mixed  $member
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function smove(mixed $srcset, mixed $dstset, mixed $member): Relay|bool;

    /**
     * Removes and returns one or more random members from the set value store at `$key`.
     *
     * @param  mixed  $set
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function spop(mixed $set, int $count = 1): mixed;

    /**
     * Returns one or multiple random members from a set.
     *
     * @param  mixed  $set
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function srandmember(mixed $set, int $count = 1): mixed;

    /**
     * Returns the set cardinality (number of elements) of the set stored at `$key`.
     *
     * @param  mixed  $key
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function scard(mixed $key): Relay|int|false;

    /**
     * Execute a script management command.
     *
     * @param  string  $command
     * @param  string  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function script(string $command, string ...$args): mixed;

    /**
     * Returns the length of the string value stored at `$key`.
     *
     * @param  mixed  $key
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function strlen(mixed $key): Relay|int|false;

    /**
     * This command swaps two Redis databases,
     * so that immediately all the clients connected to a given database
     * will see the data of the other database, and the other way around.
     *
     * @param  int  $index1
     * @param  int  $index2
     * @return Relay|bool
     */
    #[Attributes\RedisCommand]
    public function swapdb(int $index1, int $index2): Relay|bool;

    /**
     * Returns the number of fields contained in the hash stored at `$key`.
     *
     * @param  mixed  $key
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hlen(mixed $key): Relay|int|false;

    /**
     * Returns the length of the list stored at `$key`.
     *
     * @param  mixed  $key
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function llen(mixed $key): Relay|int|false;

    /**
     * Acknowledge one or more IDs as having been processed by the consumer group.
     *
     * @param  mixed  $key
     * @param  string  $group
     * @param  array  $ids
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function xack(mixed $key, string $group, array $ids): Relay|int|false;

    /**
     * Append a message to a stream.
     *
     * @param  string  $key
     * @param  string  $id
     * @param  int  $maxlen
     * @param  bool  $approx
     * @param  bool  $nomkstream
     * @return Relay|string|null|false
     */
    public function xadd(
        string $key,
        string $id,
        array $values,
        int $maxlen = 0,
        bool $approx = false,
        bool $nomkstream = false
    ): Relay|string|null|false;

    /**
     * Claim ownership of stream message(s).
     *
     * @param  string  $key
     * @param  string  $group
     * @param  string  $consumer
     * @param  int  $min_idle
     * @param  array  $ids
     * @param  array  $options
     * @return Relay|array|bool
     */
    #[Attributes\RedisCommand]
    public function xclaim(
        string $key,
        string $group,
        string $consumer,
        int $min_idle,
        array $ids,
        array $options
    ): Relay|array|bool;

    /**
     * Automatically take ownership of stream message(s) by metrics
     *
     * @param  string  $key
     * @param  string  $group
     * @param  string  $consumer
     * @param  int  $min_idle
     * @param  string  $start
     * @param  int  $count
     * @param  bool  $justid
     * @return Relay|array|bool
     */
    #[Attributes\RedisCommand]
    public function xautoclaim(
        string $key,
        string $group,
        string $consumer,
        int $min_idle,
        string $start,
        int $count = -1,
        bool $justid = false
    ): Relay|bool|array;

    /**
     * Get the length of a stream.
     *
     * @param  string  $key
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function xlen(string $key): Relay|int|false;

    /**
     * Perform utility operations having to do with consumer groups
     *
     * @param  string  $operation
     * @param  mixed  $key
     * @param  string  $group
     * @param  string  $id_or_consumer
     * @param  bool  $mkstream
     * @param  int  $entries_read
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function xgroup(
        string $operation,
        mixed $key = null,
        string $group = null,
        string $id_or_consumer = null,
        bool $mkstream = false,
        int $entries_read = -2
    ): mixed;

    /**
     * Remove one or more specific IDs from a stream.
     *
     * @param  string  $key
     * @param  array  $ids
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function xdel(string $key, array $ids): Relay|int|false;

    /**
     * Retrieve information about a stream key.
     *
     * @param  string  $operation
     * @param  string|null  $arg1
     * @param  string|null  $arg2
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function xinfo(string $operation, ?string $arg1 = null, ?string $arg2 = null, int $count = -1): mixed;

    /**
     * Query pending entries in a stream.
     *
     * @param  string  $key
     * @param  string  $group
     * @param  string|null  $start
     * @param  string|null  $end
     * @param  int  $count
     * @param  string|null  $consumer
     * @param  int  $idle
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function xpending(
        string $key,
        string $group,
        ?string $start = null,
        ?string $end = null,
        int $count = -1,
        ?string $consumer = null,
        int $idle = 0
    ): Relay|array|false;

    /**
     * Lists elements in a stream.
     *
     * @param  mixed  $key
     * @param  string  $start
     * @param  string  $end
     * @param  int  $count = -1
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function xrange(mixed $key, string $start, string $end, int $count = -1): Relay|array|false;

    /**
     * Get a range of entries from a STREAM ke in reverse chronological order.
     *
     * @param  string  $key
     * @param  string  $end
     * @param  string  $start
     * @param  int  $count
     * @return Relay|array|bool
     */
    #[Attributes\RedisCommand]
    public function xrevrange(string $key, string $end, string $start, int $count = -1): Relay|array|bool;

    /**
     * Read messages from a stream.
     *
     * @param  array  $streams
     * @param  int  $count
     * @param  int  $block
     * @return Relay|array|bool|null
     */
    #[Attributes\RedisCommand]
    public function xread(array $streams, int $count = -1, int $block = -1): Relay|array|bool|null;

    /**
     * Read messages from a stream using a consumer group.
     *
     * @param  string  $group
     * @param  string  $consumer
     * @param  array  $streams
     * @param  int  $count
     * @param  int  $block
     * @return Relay|array|bool|null
     */
    #[Attributes\RedisCommand]
    public function xreadgroup(
        string $group,
        string $consumer,
        array $streams,
        int $count = 1,
        int $block = 1
    ): Relay|array|bool|null;

    /**
     * Truncate a STREAM key in various ways.
     *
     * @param  string  $key
     * @param  string  $threshold
     * @param  bool  $approx
     * @param  bool  $minid
     * @param  int  $limit
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function xtrim(
        string $key,
        string $threshold,
        bool $approx = false,
        bool $minid = false,
        int $limit = -1
    ): Relay|int|false;

    /**
     * Adds all the specified members with the specified scores to the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function zadd(mixed $key, mixed ...$args): mixed;

    /**
     * When called with just the key argument, return a random element from the sorted set value stored at key.
     * If the provided count argument is positive, return an array of distinct elements.
     *
     * @param  mixed  $key
     * @param  array|null  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function zrandmember(mixed $key, ?array $options = null): mixed;

    /**
     * Returns the specified range of elements in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  string|int  $start
     * @param  string|int  $end
     * @param  mixed  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function zrange(mixed $key, string|int $start, string|int $end, mixed $options = null): Relay|array|false;

    /**
     * Returns the specified range of elements in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @param  mixed  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zrevrange(mixed $key, int $start, int $end, mixed $options = null): Relay|array|false;

    /**
     * Returns all the elements in the sorted set at key with a score between
     * min and max (including elements with score equal to min or max).
     *
     * @param  mixed  $key
     * @param  mixed  $start
     * @param  mixed  $end
     * @param  mixed  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zrangebyscore(mixed $key, mixed $start, mixed $end, mixed $options = null): Relay|array|false;

    /**
     * Returns all the elements in the sorted set at key with a score between
     * max and min (including elements with score equal to max or min).
     *
     * @param  mixed  $key
     * @param  mixed  $start
     * @param  mixed  $end
     * @param  mixed  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zrevrangebyscore(mixed $key, mixed $start, mixed $end, mixed $options = null): Relay|array|false;

    /**
     * Returns all the elements in the sorted set at key with a score between
     * max and min (including elements with score equal to max or min).
     *
     * @param  mixed  $dst
     * @param  mixed  $src
     * @param  mixed  $start
     * @param  mixed  $end
     * @param  mixed  $options
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zrangestore(mixed $dst, mixed $src, mixed $start, mixed $end, mixed $options = null): Relay|int|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command returns all
     * the elements in the sorted set at key with a value between min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @param  int  $offset
     * @param  int  $count
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zrangebylex(mixed $key, mixed $min, mixed $max, int $offset = -1, int $count = -1): Relay|array|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command returns all
     * the elements in the sorted set at key with a value between max and min.
     *
     * @param  mixed  $key
     * @param  mixed  $max
     * @param  mixed  $min
     * @param  int  $offset
     * @param  int  $count
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zrevrangebylex(mixed $key, mixed $max, mixed $min, int $offset = -1, int $count = -1): Relay|array|false;

    /**
     * Returns the rank of member in the sorted set stored at key, with the scores
     * ordered from low to high. The rank (or index) is 0-based, which means
     * that the member with the lowest score has rank 0.
     *
     * @param  mixed  $key
     * @param  mixed  $rank
     * @param  bool  $withscore
     * @return Relay|array|int|null|false
     */
    #[Attributes\RedisCommand]
    public function zrank(mixed $key, mixed $rank, bool $withscore = false): Relay|array|int|null|false;

    /**
     * Returns the rank of member in the sorted set stored at key, with the scores
     * ordered from high to low. The rank (or index) is 0-based, which means
     * that the member with the highest score has rank 0.
     *
     * @param  mixed  $key
     * @param  mixed  $rank
     * @param  bool  $withscore
     * @return Relay|array|int|null|false
     */
    #[Attributes\RedisCommand]
    public function zrevrank(mixed $key, mixed $rank, bool $withscore = false): Relay|array|int|null|false;

    /**
     * Removes the specified members from the sorted set stored at key.
     * Non existing members are ignored.
     *
     * @param  mixed  $key
     * @param  mixed  $args,...
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zrem(mixed $key, mixed ...$args): Relay|int|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command removes all
     * elements in the sorted set stored at key between the
     * lexicographical range specified by min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zremrangebylex(mixed $key, mixed $min, mixed $max): Relay|int|false;

    /**
     * Removes all elements in the sorted set stored at key with rank between
     * start and stop. Both start and stop are 0 -based indexes with 0 being
     * the element with the lowest score.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zremrangebyrank(mixed $key, int $start, int $end): Relay|int|false;

    /**
     * Removes all elements in the sorted set stored at key with
     * a score between min and max (inclusive).
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zremrangebyscore(mixed $key, mixed $min, mixed $max): Relay|int|false;

    /**
     * Returns the sorted set cardinality (number of elements) of the sorted set stored at key.
     *
     * @param  mixed  $key
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function zcard(mixed $key): Relay|int|false;

    /**
     * Returns the number of elements in the sorted set at key with a score between min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zcount(mixed $key, mixed $min, mixed $max): Relay|int|false;

    /**
     * This command is similar to ZDIFFSTORE, but instead of storing the
     * resulting sorted set, it is returned to the client.
     *
     * @param  array  $keys
     * @param  array  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zdiff(array $keys, ?array $options = null): Relay|array|false;

    /**
     * Computes the difference between the first and all successive
     * input sorted sets and stores the result in destination.
     *
     * @param  mixed  $dst
     * @param  array  $keys
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zdiffstore(mixed $dst, array $keys): Relay|int|false;

    /**
     * Increments the score of member in the sorted set stored at key by increment.
     *
     * @param  mixed  $key
     * @param  float  $score
     * @param  mixed  $mem
     * @return Relay|float|false
     */
    #[Attributes\RedisCommand]
    public function zincrby(mixed $key, float $score, mixed $mem): Relay|float|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command returns the
     * number of elements in the sorted set at key with a value between min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zlexcount(mixed $key, mixed $min, mixed $max): Relay|int|false;

    /**
     * Returns the scores associated with the specified members in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $mems,...
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zmscore(mixed $key, mixed ...$mems): Relay|array|false;

    /**
     * Returns the score of member in the sorted set at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @return Relay|float|null|false
     */
    #[Attributes\RedisCommand]
    public function zscore(mixed $key, mixed $member): Relay|float|null|false;

    /**
     * This command is similar to ZINTERSTORE, but instead of storing
     * the resulting sorted set, it is returned to the client.
     *
     * @param  array  $keys
     * @param  array  $weights
     * @param  mixed  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zinter(array $keys, ?array $weights = null, mixed $options = null): Relay|array|false;

    /**
     * Intersect multiple sorted sets and return the cardinality of the result.
     *
     * @param  array  $keys
     * @param  int  $limit
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zintercard(array $keys, int $limit = -1): Relay|int|false;

    /**
     * Computes the intersection of numkeys sorted sets given by the
     * specified keys, and stores the result in destination.
     *
     * @param  mixed  $dst
     * @param  array  $keys
     * @param  array  $weights
     * @param  mixed  $options
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zinterstore(mixed $dst, array $keys, ?array $weights = null, mixed $options = null): Relay|int|false;

    /**
     * This command is similar to ZUNIONSTORE, but instead of storing
     * the resulting sorted set, it is returned to the client.
     *
     * @param  array  $keys
     * @param  array  $weights
     * @param  mixed  $options
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zunion(array $keys, ?array $weights = null, mixed $options = null): Relay|array|false;

    /**
     * Computes the union of numkeys sorted sets given by the
     * specified keys, and stores the result in destination.
     *
     * @param  mixed  $dst
     * @param  array  $keys
     * @param  array  $weights
     * @param  mixed  $options
     * @return Relay|int|false
     */
    #[Attributes\RedisCommand]
    public function zunionstore(mixed $dst, array $keys, ?array $weights = null, mixed $options = null): Relay|int|false;

    /**
     * Removes and returns up to count members with the lowest
     * scores in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zpopmin(mixed $key, int $count = 1): Relay|array|false;

    /**
     * Removes and returns up to count members with the highest
     * scores in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return Relay|array|false
     */
    #[Attributes\RedisCommand]
    public function zpopmax(mixed $key, int $count = 1): Relay|array|false;

    /**
     * Returns keys cached in runtime memory.
     *
     * @internal Temporary debug helper. Do not use.
     * @return mixed
     */
    #[Attributes\Local]
    public function _getKeys();

    /**
     * Returns information about the license.
     *
     * @return array
     */
    #[Attributes\Local]
    public static function license(): array;
}

/**
 * Generic Relay exception.
 */
class Exception extends \Exception
{
}

/**
 * Relay Sentinel client.
 *
 * @see https://redis.io/docs/management/sentinel/
 */
class Sentinel
{
    /**
     * Whether to throw an exception on `-ERR` replies.  Default: false
     *
     * @var int
     */
    public const OPT_THROW_ON_ERR = 1;

    /**
     * Whether \Relay\Sentinel should automatically discover other sentinels in the
     * cluster, so it may use them if we fail to communicate with the first one.
     *
     * @var int
     */
    public const OPT_AUTO_DISCOVER = 2;

    /**
     * Establishes a new connection to a Sentinel instance.
     *
     * For backwards compatibility with PhpRedis 6.x, the
     * constructor may be called with a single options array.
     *
     * @param  array|string|null  $host
     * @param  int  $port
     * @param  float  $timeout
     * @param  mixed  $persistent
     * @param  int  $retry_interval
     * @param  float  $read_timeout
     * @param  mixed  $auth
     */
    #[Attributes\Server]
    public function __construct(
        array|string|null $host = null,
        int $port = 26379,
        float $timeout = 0,
        mixed $persistent = null,
        int $retry_interval = 0,
        float $read_timeout = 0,
        #[\SensitiveParameter]
        mixed $auth = null
    );

    /**
     * Check if the current Sentinel configuration is able to reach the quorum needed
     * to failover a master, and the majority needed to authorize the failover.
     *
     * @param  string  $master
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function ckquorum(string $master): bool;

    /**
     * Force a failover as if the master was not reachable,
     * and without asking for agreement to other Sentinels.
     *
     * @param  string  $master
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function failover(string $master): bool;

    /**
     * Force Sentinel to rewrite its configuration on disk,
     * including the current Sentinel state.
     *
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function flushconfig(): bool;

    /**
     * Returns the ip and port number of the master with that name.
     *
     * @param  string  $master
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function getMasterAddrByName(string $master): array|false;

    /**
     * Returns the state and info of the specified master.
     *
     * @param  string  $master
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function master(string $master): array|false;

    /**
     * Returns a list of monitored masters and their state.
     *
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function masters(): array|false;

    /**
     * Returns the ID of the Sentinel instance.
     *
     * @return string
     */
    #[Attributes\RedisCommand]
    public function myid(): string;

    /**
     * Returns PONG if no message is provided, otherwise returns the message.
     *
     * @param  string|null  $message
     * @return string|bool
     */
    #[Attributes\RedisCommand]
    public function ping(?string $message = null): string|bool;

    /**
     * Will reset all the masters with matching name.
     *
     * @param  string  $pattern
     * @return int
     */
    #[Attributes\RedisCommand]
    public function reset(string $pattern): int;

    /**
     * Returns a list of sentinel instances for this master, and their state.
     *
     * @param  string  $master
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function sentinels(string $master): array|false;

    /**
     * Show a list of replicas for this master, and their state.
     *
     * @param  string  $master
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function slaves(string $master): array|false;

    /**
     * Returns the last error message, if any.
     *
     * @return string|null
     */
    #[Attributes\Local]
    public function getLastError(): string|null;

    /**
     * Sets a client option.
     *
     * @param  int  $option
     * @param  mixed  $value
     * @return bool
     */
    #[Attributes\Local]
    public function setOption(int $option, mixed $value): bool;

    /**
     * Returns a client option.
     *
     * @param  int  $option
     * @return mixed
     */
    #[Attributes\Local]
    public function getOption(int $option): mixed;
}

/**
 * Relay Table is a persistent per-worker hash table that can store arbitrary data.
 */
class Table
{
    /**
     * Create a Relay table instance.
     *
     * @param  string|null  $namespace
     * @param  int  $serializer
     */
    public function __construct(?string $namespace = null, int $serializer = Relay::SERIALIZER_PHP);

    /**
     * Returns a key, or `null` if key doesn't exist.
     *
     * @param  string  $key
     * @return mixed
     */
    public function get(string $key): mixed;

    /**
     * Pluck a key from a cached key.
     *
     * @param  string  $key
     * @param  string  $field
     * @return mixed
     */
    public function pluck(string $key, string $field): mixed;

    /**
     * Set a key and its value.
     *
     * @param  string  $key
     * @param  mixed  $value;
     * @return bool
     */
    public function set(string $key, mixed $value): bool;

    /**
     * Check if a key exists in the table.
     *
     * @param  string  $key
     * @return bool
     */
    public function exists(string $key): bool;

    /**
     * Remove a key from the table.
     *
     * @param  string  $key
     * @return bool
     */
    public function delete(string $key): bool;

    /**
     * The number of keys stored in the table.
     *
     * @return int
     */
    public function count(): int;

    /**
     * Get the table's namespace.
     *
     * @return string|null
     */
    public function namespace(): string|null;

    /**
     * Returns all table namespaces.
     *
     * @return array
     */
    public static function namespaces(): array;

    /**
     * Removes all keys from the table.
     *
     * @return bool
     */
    public function clear(): bool;

    /**
     * Removes all keys from all tables.
     *
     * @return int
     */
    public static function clearAll(): int;
}

/**
 * Relay Cluster client.
 *
 * @see https://redis.io/docs/management/scaling/
 */
class Cluster
{
    /**
     * Integer representing the failover option.
     *
     * @var int
     */
    public const OPT_SLAVE_FAILOVER = 5;

    /**
     * Integer representing no failover.
     *
     * Enabled by default. Send commands to master nodes only.
     *
     * @var int
     */
    public const FAILOVER_NONE = 0;

    /**
     * Integer representing error failover.
     *
     * Send readonly commands to slave nodes if master is unreachable.
     *
     * @var int
     */
    public const FAILOVER_ERROR = 1;

    /**
     * Integer representing distribute failover.
     *
     * Always distribute readonly commands between master and slaves, at random
     *
     * @var int
     */
    public const FAILOVER_DISTRIBUTE = 2;

    /**
     * Integer representing distribute slaves failover.
     *
     * Always distribute readonly commands to the slaves, at random
     *
     * @var int
     */
    public const FAILOVER_DISTRIBUTE_SLAVES = 3;

    /**
     * Create a cluster object.
     *
     * @param  string|null  $name
     * @param  array|null  $seeds
     * @param  int|float  $connect_timeout
     * @param  int|float  $command_timeout
     * @param  bool  $persistent
     * @param  mixed  $auth
     * @param  array|null  $context
     */
    #[Attributes\Server]
    public function __construct(
        string|null $name,
        array|null $seeds = null,
        int|float $connect_timeout = 0,
        int|float $command_timeout = 0,
        bool $persistent = false,
        #[\SensitiveParameter]
        mixed $auth = null,
        array|null $context = null
    );

    /**
     * Compress data with Relay's currently configured compression algorithm.
     *
     * @param  string  $value
     * @return string
     */
    #[Attributes\Local]
    public function _compress(string $value): string;

    /**
     * Returns the number of milliseoconds since Relay has received a reply from the cluster.
     *
     * @return int
     */
    #[Attributes\Local]
    public function idleTime(): int;

    /**
     * Returns an array of endpoints along with each of their keys cached in runtime memory.
     *
     * @internal Temporary debug helper. Do not use.
     * @return array|false
     */
    public function _getKeys(): array|false;

    /**
     * Return a list of master nodes
     *
     * @return array
     */
    #[Attributes\Local]
    public function _masters(): array;

    /**
     * Returns the serialized and compressed value.
     *
     * @param  mixed  $value
     * @return string
     */
    #[Attributes\Local]
    public function _pack(mixed $value): string;

    /**
     * Returns the value with the prefix.
     *
     * @param  mixed  $value
     * @return string
     */
    #[Attributes\Local]
    public function _prefix(mixed $value): string;

    /**
     * Returns the serialized value.
     *
     * @param  mixed  $value
     * @return string
     */
    #[Attributes\Local]
    public function _serialize(mixed $value): string;

    /**
     * Uncompress data with Relay's currently configured compression algorithm.
     *
     * @param  string  $value
     * @return string
     */
    #[Attributes\Local]
    public function _uncompress(string $value): string;

    /**
     * Returns the unserialized and decompressed value.
     *
     * @param  string  $value
     * @return mixed
     */
    #[Attributes\Local]
    public function _unpack(string $value): mixed;

    /**
     * Returns the unserialized value.
     *
     * @param  string  $value
     * @return mixed
     */
    #[Attributes\Local]
    public function _unserialize(string $value): mixed;

    /**
     * Interact with Redis' ACLs
     *
     * @param  string  $operation
     * @param  string  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function acl(array|string $key_or_address, string $operation, string ...$args): mixed;

    /**
     * Adds allow pattern(s). Only matching keys will be cached in memory.
     *
     * @param  string  $pattern,...
     * @return int
     */
    #[Attributes\Local]
    public function addAllowPatterns(string ...$pattern): int;

    /**
     * Adds ignore pattern(s). Matching keys will not be cached in memory.
     *
     * @param  string  $pattern,...
     * @return int
     */
    #[Attributes\Local]
    public function addIgnorePatterns(string ...$pattern): int;

    /**
     * If key already exists and is a string, this command appends
     * the value at the end of the string. If key does not exist
     * it is created and set as an empty string, so APPEND will
     * be similar to SET in this special case.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function append(mixed $key, mixed $value): Cluster|int|false;

    /**
     * Asynchronously rewrite the append-only file.
     *
     * @param  array|string  $key_or_address
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function bgrewriteaof(array|string $key_or_address): Cluster|bool;

    /**
     * Paus the client until sufficient local and/or remote AOF data has been flushed to disk.
     *
     * @param  array|string  $key_or_address
     * @param  int  $numlocal
     * @param  int  $numremote
     * @return Relay|array
     */
    #[Attributes\RedisCommand]
    public function waitaof(array|string $key_or_address, int $numlocal, int $numremote, int $timeout): Relay|array|false;

    /**
     * Asynchronously save the dataset to disk.
     *
     * @param  array|string  $key_or_address
     * @param  bool  $schedule
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function bgsave(array|string $key_or_address, bool $schedule = false): Cluster|bool;

    /**
     * Count the number of set bits (population counting) in a string.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @param  bool  $by_bit
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function bitcount(mixed $key, int $start = 0, int $end = -1, bool $by_bit = false): Cluster|int|false;

    /**
     * Perform a bitwise operation on one or more keys, storing the result in a new key.
     *
     * @param  string  $operation
     * @param  string  $dstkey
     * @param  string  $srckey
     * @param  string  $other_keys,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function bitop(string $operation, string $dstkey, string $srckey, string ...$other_keys): Cluster|int|false;

    /**
     * Return the position of the first bit set to 1 or 0 in a string.
     *
     * @param  mixed  $key
     * @param  int  $bit
     * @param  int  $start
     * @param  int  $end
     * @param  bool  $by_bit
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function bitpos(mixed $key, int $bit, int $start = null, int $end = null, bool $by_bit = false): Cluster|int|false;

    /**
     * BLMOVE is the blocking variant of LMOVE. When source contains elements,
     * this command behaves exactly like LMOVE. When used inside a
     * MULTI/EXEC block, this command behaves exactly like LMOVE.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @param  string  $srcpos
     * @param  string  $dstpos
     * @param  float  $timeout
     * @return Cluster|string|null|false
     */
    #[Attributes\RedisCommand]
    public function blmove(mixed $srckey, mixed $dstkey, string $srcpos, string $dstpos, float $timeout): Cluster|string|null|false;

    /**
     * Pop elements from a list, or block until one is available
     *
     * @param  float  $timeout
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function blmpop(float $timeout, array $keys, string $from, int $count = 1): mixed;

    /**
     * BLPOP is a blocking list pop primitive. It is the blocking version of LPOP because
     * it blocks the connection when there are no elements to pop from any of the given lists.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function blpop(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Cluster|array|null|false;

    /**
     * BRPOP is a blocking list pop primitive. It is the blocking version of RPOP because
     * it blocks the connection when there are no elements to pop from any of the given lists.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function brpop(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Cluster|array|null|false;

    /**
     * Atomically returns and removes the last element (tail) of the list stored at source,
     * and pushes the element at the first element (head) of the list stored at destination.
     * This command will block for an element up to the provided timeout.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @param  float  $timeout
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function brpoplpush(mixed $srckey, mixed $dstkey, float $timeout): mixed;

    /**
     * Remove and return members with scores in a sorted set or block until one is available
     *
     * @param  float  $timeout
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function bzmpop(float $timeout, array $keys, string $from, int $count = 1): Cluster|array|null|false;

    /**
     * BZPOPMAX is the blocking variant of the sorted set ZPOPMAX primitive.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function bzpopmax(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Cluster|array|null|false;

    /**
     * BZPOPMIN is the blocking variant of the sorted set ZPOPMIN primitive.
     *
     * @param  string|array  $key
     * @param  string|float  $timeout_or_key
     * @param  array  $extra_args,...
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function bzpopmin(string|array $key, string|float $timeout_or_key, mixed ...$extra_args): Cluster|array|null|false;

    /**
     * Clears the last error that is set, if any.
     *
     * @return bool
     */
    #[Attributes\Local]
    public function clearLastError(): bool;

    /**
     * @return bool
     */
    #[Attributes\Local]
    public function clearTransferredBytes(): bool;

    /**
     * Executes `CLIENT` command operations.
     *
     * @param  array|string  $key_or_address
     * @param  string  $operation
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function client(array|string $key_or_address, string $operation, mixed ...$args): mixed;

    /**
     * Closes the current connection, if it's persistent.
     *
     * @return bool
     */
    #[Attributes\Local]
    public function close(): bool;

    /**
     * Executes `CLUSTER` command operations.
     *
     * @param  array|string  $key_or_address
     * @param  string  $operation
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function cluster(array|string $key_or_address, string $operation, mixed ...$args): mixed;

    /**
     * This is a container command for runtime configuration commands.
     *
     * @param  array|string  $key_or_address
     * @param  string  $operation
     * @param  mixed  $args,...
     * @return Cluster|array|bool
     */
    #[Attributes\RedisCommand]
    public function config(array|string $key_or_address, string $operation, mixed ...$args): mixed;

    /**
     * Return an array with details about every Redis command.
     *
     * @param  array  $args,...
     * @return Cluster|array|int|false
     */
    #[Attributes\RedisCommand]
    public function command(mixed ...$args): Cluster|array|int|false;

    /**
     * This command copies the value stored at the source key to the destination key.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @param  array|null  $options
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function copy(mixed $srckey, mixed $dstkey, array|null $options = null): Cluster|bool;

    /**
     * Returns the number of keys in the currently-selected database.
     *
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function dbsize(array|string $key_or_address): Cluster|int|false;

    /**
     * Decrements the number stored at key by one.
     *
     * @param  mixed  $key
     * @param  int  $by
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function decr(mixed $key, int $by = 1): Cluster|int|false;

    /**
     * Decrements the number stored at key by decrement.
     *
     * @param  mixed  $key
     * @param  int  $value
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function decrby(mixed $key, int $value): Cluster|int|false;

    /**
     * Removes the specified keys.
     *
     * @param  mixed  $keys,...
     * @return Cluster|int|bool
     */
    #[Attributes\RedisCommand]
    public function del(mixed ...$keys): Cluster|int|bool;

    /**
     * Flushes all previously queued commands in a transaction and restores the connection state to normal.
     * If WATCH was used, DISCARD unwatches all keys watched by the connection.
     *
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function discard(): bool;

    /**
     * Dispatches all pending events.
     *
     * @return int|false
     */
    #[Attributes\Local]
    public function dispatchEvents(): int|false;

    /**
     * Serialize and return the value stored at key in a Redis-specific format.
     *
     * @param  mixed  $key
     * @return Cluster|string|false
     */
    #[Attributes\RedisCommand]
    public function dump(mixed $key): Cluster|string|false;

    /**
     * Asks Redis to echo back the provided string.
     *
     * @param  array|string  $key_or_address
     * @param  string  $message
     * @return Cluster|string|false
     */
    #[Attributes\RedisCommand]
    public function echo(array|string $key_or_address, string $message): Cluster|string|false;

    /**
     * Returns the connection's endpoint identifier.
     *
     * @return array|false
     */
    #[Attributes\Local]
    public function endpointId(): array|false;

    /**
     * Evaluate script using the Lua interpreter.
     *
     * @see https://redis.io/commands/eval
     *
     * @param  mixed  $script
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function eval(mixed $script, array $args = [], int $num_keys = 0): mixed;

    /**
     * Evaluate script using the Lua interpreter.  This is just the "read-only" variant of EVAL
     * meaning it can be run on read-only replicas.
     *
     * @see https://redis.io/commands/eval_ro
     *
     * @param  mixed  $script
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function eval_ro(mixed $script, array $args = [], int $num_keys = 0): mixed;

    /**
     * Evaluates a script cached on the server-side by its SHA1 digest.
     *
     * @param  string  $sha
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function evalsha(string $sha, array $args = [], int $num_keys = 0): mixed;

    /**
     * Evaluates a script cached on the server-side by its SHA1 digest.  This is just the "read-only" variant
     * of `EVALSHA` meaning it can be run on read-only replicas.
     *
     * @param  string  $sha
     * @param  array  $args
     * @param  int  $num_keys
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function evalsha_ro(string $sha, array $args = [], int $num_keys = 0): mixed;

    /**
     * Executes all previously queued commands in a transaction and restores the connection state to normal.
     *
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function exec(): array|false;

    /**
     * Returns if key(s) exists.
     *
     * @param  mixed  $keys,...
     * @return Cluster|int|bool
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function exists(mixed ...$keys): Cluster|int|bool;

    /**
     * Set a timeout on key.
     *
     * @param  mixed  $key
     * @param  int  $seconds
     * @param  string|null  $mode
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function expire(mixed $key, int $seconds, string|null $mode = null): Cluster|bool;

    /**
     * Set a timeout on key using a unix timestamp.
     *
     * @param  mixed  $key
     * @param  int  $timestamp
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function expireat(mixed $key, int $timestamp): Cluster|bool;

    /**
     * Returns the absolute Unix timestamp in seconds at which the given key will expire.
     * If the key exists but doesn't have a TTL this function return -1.
     * If the key does not exist -2.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function expiretime(mixed $key): Cluster|int|false;

    /**
     * @see \Relay\Relay::flushMemory()
     *
     * @param  string|null  $endpointId
     * @param  int|null  $db
     * @return bool
     */
    #[Attributes\Local]
    public static function flushMemory(?string $endpointId = null, int $db = null): bool;

    /**
     * Deletes all the keys of all the existing databases, not just the currently selected one.
     *
     * @param  array|string  $key_or_address
     * @param  bool|null  $sync
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function flushall(array|string $key_or_address, bool|null $sync = null): Cluster|bool;

    /**
     * Deletes all the keys of the currently selected database.
     *
     * @param  array|string  $key_or_address
     * @param  bool|null  $sync
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function flushdb(array|string $key_or_address, bool|null $sync = null): Cluster|bool;

    /**
     * Add one or more members to a geospacial sorted set
     *
     * @param  mixed  $key
     * @param  float  $lng
     * @param  float  $lat
     * @param  string  $member
     * @param  mixed  $other_triples_and_options,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function geoadd(mixed $key, float $lng, float $lat, string $member, mixed ...$other_triples_and_options): Cluster|int|false;

    /**
     * Get the distance between two members of a geospacially encoded sorted set.
     *
     * @param  mixed  $key
     * @param  string  $src
     * @param  string  $dst
     * @param  string|null  $unit
     * @return Cluster|float|false
     */
    #[Attributes\RedisCommand]
    public function geodist(mixed $key, string $src, string $dst, string|null $unit = null): Cluster|float|false;

    /**
     * Retrieve one or more GeoHash encoded strings for members of the set.
     *
     * @param  mixed  $key
     * @param  string  $member
     * @param  string  $other_members,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function geohash(mixed $key, string $member, string ...$other_members): Cluster|array|false;

    /**
     * Return the positions (longitude,latitude) of all the specified members
     * of the geospatial index represented by the sorted set at key.
     *
     * @param  mixed  $key
     * @param  mixed  $members,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function geopos(mixed $key, mixed ...$members): Cluster|array|false;

    /**
     * Retrieve members of a geospacially sorted set that are within a certain radius of a location.
     *
     * @param  mixed  $key
     * @param  float  $lng
     * @param  float  $lat
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadius(mixed $key, float $lng, float $lat, float $radius, string $unit, array $options = []): mixed;

    /**
     * Retrieve members of a geospacially sorted set that are within a certain radius of a location.
     *
     * @param  mixed  $key
     * @param  float  $lng
     * @param  float  $lat
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadius_ro(mixed $key, float $lng, float $lat, float $radius, string $unit, array $options = []): mixed;

    /**
     * Similar to `GEORADIUS` except it uses a member as the center of the query.
     *
     * @param  mixed  $key
     * @param  string  $member
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadiusbymember(mixed $key, string $member, float $radius, string $unit, array $options = []): mixed;

    /**
     * Similar to `GEORADIUS` except it uses a member as the center of the query.
     *
     * @param  mixed  $key
     * @param  string  $member
     * @param  float  $radius
     * @param  string  $unit
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function georadiusbymember_ro(mixed $key, string $member, float $radius, string $unit, array $options = []): mixed;

    /**
     * Search a geospacial sorted set for members in various ways.
     *
     * @param  mixed  $key
     * @param  array|string  $position
     * @param  array|int|float  $shape
     * @param  string  $unit
     * @param  array  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function geosearch(mixed $key, array|string $position, array|int|float $shape, string $unit, array $options = []): Cluster|array|false;

    /**
     * Search a geospacial sorted set for members within a given area or range, storing the results into
     * a new set.
     *
     * @param  mixed  $dstkey
     * @param  mixed  $srckey
     * @param  array|string  $position
     * @param  array|int|float  $shape
     * @param  string  $unit
     * @param  array  $options
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function geosearchstore(mixed $dstkey, mixed $srckey, array|string $position, array|int|float $shape, string $unit, array $options = []): Cluster|int|false;

    /**
     * Get the value of key.
     *
     * @param  mixed  $key
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function get(mixed $key): mixed;

    /**
     * Returns the bit value at offset in the string value stored at key.
     *
     * @param  mixed  $key
     * @param  int  $pos
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function getbit(mixed $key, int $pos): Cluster|int|false;

    /**
     * Get the value of key and optionally set its expiration.
     * GETEX is similar to GET, but is a write command with additional options.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function getex(mixed $key, ?array $options = null): mixed;

    /**
     * Returns the last error message, if any.
     *
     * @return string|null
     */
    #[Attributes\Local]
    public function getLastError(): string|null;

    /**
     * Get the mode Relay is currently in.
     * `Relay::ATOMIC` or `Relay::MULTI`.
     *
     * @param  bool  $masked
     * @return int
     */
    #[Attributes\Local]
    public function getMode(bool $masked = false): int;

    /**
     * Returns a client option.
     *
     * @param  int  $option
     * @return mixed
     */
    #[Attributes\Local]
    public function getOption(int $option): mixed;

    /**
     * @return array|false
     */
    #[Attributes\Local]
    public function getTransferredBytes(): array|false;

    /**
     * Returns the substring of the string value stored at key,
     * determined by the offsets start and end (both are inclusive).
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @return Cluster|string|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function getrange(mixed $key, int $start, int $end): Cluster|string|false;

    /**
     * Atomically sets key to value and returns the old value stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function getset(mixed $key, mixed $value): mixed;

    /**
     * Removes the specified fields from the hash stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  string  $members,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function hdel(mixed $key, mixed $member, mixed ...$members): Cluster|int|false;

    /**
     * Returns if field is an existing field in the hash stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hexists(mixed $key, mixed $member): Cluster|bool;

    /**
     * Returns the value associated with field in the hash stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hget(mixed $key, mixed $member): mixed;

    /**
     * Returns all fields and values of the hash stored at key.
     *
     * @param  mixed  $key
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hgetall(mixed $key): Cluster|array|false;

    /**
     * Increments the number stored at field in the hash stored at key by increment.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  int  $value
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function hincrby(mixed $key, mixed $member, int $value): Cluster|int|false;

    /**
     * Increment the specified field of a hash stored at key, and representing
     * a floating point number, by the specified increment.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  float  $value
     * @return Cluster|float|bool
     */
    #[Attributes\RedisCommand]
    public function hincrbyfloat(mixed $key, mixed $member, float $value): Cluster|float|bool;

    /**
     * Returns all field names in the hash stored at key.
     *
     * @param  mixed  $key
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hkeys(mixed $key): Cluster|array|false;

    /**
     * Returns the number of fields contained in the hash stored at `$key`.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hlen(mixed $key): Cluster|int|false;

    /**
     * Returns the values associated with the specified fields in the hash stored at key.
     *
     * @param  mixed  $key
     * @param  array  $members
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hmget(mixed $key, array $members): Cluster|array|false;

    /**
     * Sets the specified fields to their respective values in the hash stored at key.
     *
     * @param  mixed  $key
     * @param  array  $members
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function hmset(mixed $key, array $members): Cluster|bool;

    /**
     * When called with just the key argument, return a random field from the hash value stored at key.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @return Cluster|array|string|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hrandfield(mixed $key, array|null $options = null): Cluster|array|string|false;

    /**
     * Iterates fields of Hash types and their associated values.
     *
     * @param  mixed  $key
     * @param  mixed  $iterator
     * @param  mixed  $match
     * @param  int  $count
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function hscan(mixed $key, mixed &$iterator, mixed $match = null, int $count = 0): array|false;

    /**
     * Sets field in the hash stored at key to value.
     *
     * @param  mixed  $key
     * @param  mixed  $keys_and_vals...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function hset(mixed $key, mixed ...$keys_and_vals): Cluster|int|false;

    /**
     * Sets field in the hash stored at key to value, only if field does not yet exist.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  mixed  $value
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function hsetnx(mixed $key, mixed $member, mixed $value): Cluster|bool;

    /**
     * Returns the string length of the value associated with field in the hash stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hstrlen(mixed $key, mixed $member): Cluster|int|false;

    /**
     * Returns all values in the hash stored at key.
     *
     * @param  mixed  $key
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function hvals(mixed $key): Cluster|array|false;

    /**
     * Increments the number stored at key by one.
     *
     * @param  mixed  $key
     * @param  int  $by
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function incr(mixed $key, int $by = 1): Cluster|int|false;

    /**
     * Increments the number stored at key by increment.
     *
     * @param  mixed  $key
     * @param  int  $value
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function incrby(mixed $key, int $value): Cluster|int|false;

    /**
     * Increment the string representing a floating point number stored at key by the specified increment.
     *
     * @param  mixed  $key
     * @param  float  $value
     * @return Cluster|float|false
     */
    #[Attributes\RedisCommand]
    public function incrbyfloat(mixed $key, float $value): Cluster|float|false;

    /**
     * The INFO command returns information and statistics about Redis in a format
     * that is simple to parse by computers and easy to read by humans.
     *
     * @see https://redis.io/commands/info
     *
     * @param  array|string  $key_or_address
     * @param  string  $sections,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function info(array|string $key_or_address, string ...$sections): Cluster|array|false;

    /**
     * Returns all keys matching pattern.
     *
     * @param  mixed  $pattern
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function keys(mixed $pattern): Cluster|array|false;

    /**
     * Returns the UNIX time stamp of the last successful save to disk.
     *
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function lastsave(array|string $key_or_address): Cluster|int|false;

    /**
     * Get the longest common subsequence between two string keys.
     *
     * @param  mixed  $key1
     * @param  mixed  $key2
     * @param  array|null  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function lcs(mixed $key1, mixed $key2, array|null $options = null): mixed;

    /**
     * Returns the element at index index in the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $index
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function lindex(mixed $key, int $index): mixed;

    /**
     * Inserts element in the list stored at key either before or after the reference value pivot.
     *
     * @param  mixed  $key
     * @param  string  $op
     * @param  mixed  $pivot
     * @param  mixed  $element
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function linsert(mixed $key, string $op, mixed $pivot, mixed $element): Cluster|int|false;

    /**
     * Registers a new event listener.
     *
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\Local]
    public function listen(?callable $callback): bool;

    /**
     * Returns the length of the list stored at `$key`.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function llen(mixed $key): Cluster|int|false;

    /**
     * Atomically returns and removes the first/last element of the list
     * stored at source, and pushes the element at the first/last
     * element of the list stored at destination.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @param  string  $srcpos
     * @param  string  $dstpos
     * @return Cluster|string|null|false
     */
    #[Attributes\RedisCommand]
    public function lmove(mixed $srckey, mixed $dstkey, string $srcpos, string $dstpos): Cluster|string|null|false;

    /**
     * Pops one or more elements from the first non-empty list key from the list of provided key names.
     *
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function lmpop(array $keys, string $from, int $count = 1): mixed;

    /**
     * Removes and returns the first elements of the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function lpop(mixed $key, int $count = 1): mixed;

    /**
     * The command returns the index of matching elements inside a Redis list.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @param  array  $options
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function lpos(mixed $key, mixed $value, array|null $options = null): mixed;

    /**
     * Insert all the specified values at the head of the list stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function lpush(mixed $key, mixed $member, mixed ...$members): Cluster|int|false;

    /**
     * Inserts specified values at the head of the list stored at key,
     * only if key already exists and holds a list.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function lpushx(mixed $key, mixed $member, mixed ...$members): Cluster|int|false;

    /**
     * Returns the specified elements of the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $stop
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function lrange(mixed $key, int $start, int $stop): Cluster|array|false;

    /**
     * Removes the first count occurrences of elements equal to element from the list stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  int  $count
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function lrem(mixed $key, mixed $member, int $count = 0): Cluster|int|false;

    /**
     * Sets the list element at index to element.
     *
     * @param  mixed  $key
     * @param  int  $index
     * @param  mixed  $member
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function lset(mixed $key, int $index, mixed $member): Cluster|bool;

    /**
     * Trim an existing list so that it will contain only the specified range of elements specified.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function ltrim(mixed $key, int $start, int $end): Cluster|bool;

    /**
     * Returns the number of bytes allocated, or `0` in client-only mode.
     *
     * @return int
     */
    #[Attributes\Local]
    public static function maxMemory(): int;

    /**
     * Returns the values of all specified keys.
     *
     * @param  array  $keys
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function mget(array $keys): Cluster|array|false;

    /**
     * Sets the given keys to their respective values.
     * MSET replaces existing values with new values, just as regular SET.
     *
     * @param  array  $kvals
     * @return Cluster|array|bool
     */
    #[Attributes\RedisCommand]
    public function mset(array $kvals): Cluster|array|bool;

    /**
     * Sets the given keys to their respective values.
     * MSETNX will not perform any operation at all even if just a single key already exists.
     *
     * @param  array  $kvals
     * @return Cluster|array|bool
     */
    #[Attributes\RedisCommand]
    public function msetnx(array $kvals): Cluster|array|bool;

    /**
     * Marks the start of a transaction block. Subsequent commands will be queued for atomic execution using EXEC.
     *
     * Accepts only `Relay::MULTI` mode.
     *
     * @param  int  $mode
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function multi(int $mode = Relay::MULTI): Cluster|bool;

    /**
     * This is a container command for object introspection commands.
     *
     * @param  string  $op
     * @param  mixed  $key
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function object(string $op, mixed $key): mixed;

    /**
     * Registers a new `flushed` event listener.
     *
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\Local]
    public function onFlushed(?callable $callback): bool;

    /**
     * Registers a new `invalidated` event listener.
     *
     * @param  callable  $callback
     * @param  string|null  $pattern
     * @return bool
     */
    #[Attributes\Local]
    public function onInvalidated(?callable $callback, ?string $pattern = null): bool;

    /**
     * Remove the existing timeout on key, turning the key from volatile to persistent.
     *
     * @param  mixed  $key
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function persist(mixed $key): Cluster|bool;

    /**
     * Set a key's time to live in milliseconds.
     *
     * @param  mixed  $key
     * @param  int  $milliseconds
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function pexpire(mixed $key, int $milliseconds): Cluster|bool;

    /**
     * Set the expiration for a key as a UNIX timestamp specified in milliseconds.
     *
     * @param  mixed  $key
     * @param  int  $timestamp_ms
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function pexpireat(mixed $key, int $timestamp_ms): Cluster|bool;

    /**
     * Semantic the same as EXPIRETIME, but returns the absolute Unix expiration
     * timestamp in milliseconds instead of seconds.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function pexpiretime(mixed $key): Cluster|int|false;

    /**
     * Adds the specified elements to the specified HyperLogLog.
     *
     * @param  string  $key
     * @param  array  $elements
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function pfadd(mixed $key, array $elements): Cluster|int|false;

    /**
     * Return the approximated cardinality of the set(s) observed by the HyperLogLog at key(s).
     *
     * @param  string  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function pfcount(mixed $key): Cluster|int|false;

    /**
     * Merge given HyperLogLogs into a single one.
     *
     * @param  string  $dstkey
     * @param  array  $srckeys
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function pfmerge(string $dstkey, array $srckeys): Cluster|bool;

    /**
     * Returns PONG if no argument is provided, otherwise return a copy of the argument as a bulk.
     *
     * @param  array|string  $key_or_address
     * @param  string|null  $message
     * @return Cluster|bool|string
     */
    #[Attributes\RedisCommand]
    public function ping(array|string $key_or_address, string|null $message = null): Cluster|bool|string;

    /**
     * Set key to hold the string value and set key to timeout after a given number of milliseconds.
     *
     * @param  mixed  $key
     * @param  int  $milliseconds
     * @param  mixed  $value
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function psetex(mixed $key, int $milliseconds, mixed $value): Cluster|bool;

    /**
     * Subscribes to the given patterns.
     *
     * @param  array  $patterns
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function psubscribe(array $patterns, callable $callback): bool;

    /**
     * Returns the remaining time to live of a key that has a timeout in milliseconds.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function pttl(mixed $key): Cluster|int|false;

    /**
     * Posts a message to the given channel.
     *
     * @param  string  $channel
     * @param  string  $message
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function publish(string $channel, string $message): Cluster|int|false;

    /**
     * A container command for Pub/Sub introspection commands.
     *
     * @param  array|string  $key_or_address
     * @param  string  $operation
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function pubsub(array|string $key_or_address, string $operation, mixed ...$args): mixed;

    /**
     * Unsubscribes from the given patterns, or from all of them if none is given.
     *
     * @param  array  $patterns
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function punsubscribe(array $patterns = []): bool;

    /**
     * Returns a random key from Redis.
     *
     * @param  array|string  $key_or_address
     * @return Cluster|bool|string
     */
    #[Attributes\RedisCommand]
    public function randomkey(array|string $key_or_address): Cluster|bool|string;

    /**
     * Execute any command against Redis, without applying
     * the prefix, compression and serialization.
     *
     * @param  array|string  $key_or_address
     * @param  string  $cmd
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function rawCommand(array|string $key_or_address, string $cmd, mixed ...$args): mixed;

    /**
     * Renames key.
     *
     * @param  mixed  $key
     * @param  mixed  $newkey
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function rename(mixed $key, mixed $newkey): Cluster|bool;

    /**
     * Renames key if the new key does not yet exist.
     *
     * @param  mixed  $key
     * @param  mixed  $newkey
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function renamenx(mixed $key, mixed $newkey): Cluster|bool;

    /**
     * Create a key associated with a value that is obtained by deserializing the provided serialized value.
     *
     * @param  mixed  $key
     * @param  int  $ttl
     * @param  string  $value
     * @param  array|null  $options
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function restore(mixed $key, int $ttl, string $value, array|null $options = null): Cluster|bool;

    /**
     * Returns the role of the instance in the context of replication.
     *
     * @param  array|string  $key_or_address
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function role(array|string $key_or_address): Cluster|array|false;

    /**
     * Removes and returns the last elements of the list stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function rpop(mixed $key, int $count = 1): mixed;

    /**
     * Atomically returns and removes the last element (tail) of the list stored at source,
     * and pushes the element at the first element (head) of the list stored at destination.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function rpoplpush(mixed $srckey, mixed $dstkey): mixed;

    /**
     * Insert all the specified values at the tail of the list stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function rpush(mixed $key, mixed $member, mixed ...$members): Cluster|int|false;

    /**
     * Inserts specified values at the tail of the list stored at key,
     * only if key already exists and holds a list.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function rpushx(mixed $key, mixed $member, mixed ...$members): Cluster|int|false;

    /**
     * Add the specified members to the set stored at `$key`.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function sadd(mixed $key, mixed $member, mixed ...$members): Cluster|int|false;

    /**
     * Synchronously save the dataset to disk.
     *
     * @param  array|string  $key_or_address
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function save(array|string $key_or_address): Cluster|bool;

    /**
     * Scan the keyspace for matching keys.
     *
     * @param  mixed  $iterator
     * @param  array|string  $key_or_address
     * @param  mixed  $match
     * @param  int  $count
     * @param  string|null  $type
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function scan(mixed &$iterator, array|string $key_or_address, mixed $match = null, int $count = 0, string|null $type = null): array|false;

    /**
     * Returns the set cardinality (number of elements) of the set stored at `$key`.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function scard(mixed $key): Cluster|int|false;

    /**
     * Execute a script management command.
     *
     * @param  array|string  $key_or_address
     * @param  string  $operation
     * @param  string  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function script(array|string $key_or_address, string $operation, string ...$args): mixed;

    /**
     * Returns the members of the set resulting from the difference between the first set and all the successive sets.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sdiff(mixed $key, mixed ...$other_keys): Cluster|array|false;

    /**
     * This command is equal to SDIFF, but instead of returning the resulting set, it is stored in destination.
     * If destination already exists, it is overwritten.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function sdiffstore(mixed $key, mixed ...$other_keys): Cluster|int|false;

    /**
     * Set key to hold the string value. If key already holds
     * a value, it is overwritten, regardless of its type.
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @param  mixed  $options
     * @return Cluster|string|bool
     */
    #[Attributes\RedisCommand]
    public function set(mixed $key, mixed $value, mixed $options = null): Cluster|string|bool;

    /**
     * Sets a client option.
     *
     * Relay specific options:
     *
     * - `OPT_ALLOW_PATTERNS`
     * - `OPT_IGNORE_PATTERNS`
     * - `OPT_THROW_ON_ERROR`
     * - `OPT_CLIENT_INVALIDATIONS`
     * - `OPT_PHPREDIS_COMPATIBILITY`
     *
     * Supported PhpRedis options:
     *
     * - `OPT_PREFIX`
     * - `OPT_READ_TIMEOUT`
     * - `OPT_COMPRESSION`
     * - `OPT_COMPRESSION_LEVEL`
     * - `OPT_MAX_RETRIES`
     * - `OPT_BACKOFF_ALGORITHM`
     * - `OPT_BACKOFF_BASE`
     * - `OPT_BACKOFF_CAP`
     * - `OPT_SCAN`
     * - `OPT_REPLY_LITERAL`
     * - `OPT_NULL_MULTIBULK_AS_NULL`
     * - `OPT_SLAVE_FAILOVER`
     *
     * @param  int  $option
     * @param  mixed  $value
     * @return bool
     */
    #[Attributes\Local]
    public function setOption(int $option, mixed $value): bool;

    /**
     * Sets or clears the bit at offset in the string value stored at key.
     *
     * @param  mixed  $key
     * @param  int  $pos
     * @param  int  $value
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function setbit(mixed $key, int $pos, int $value): Cluster|int|false;

    /**
     * Set key to hold the string value and set key to timeout after a given number of seconds.
     *
     * @param  mixed  $key
     * @param  int  $seconds
     * @param  mixed  $value
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function setex(mixed $key, int $seconds, mixed $value): Cluster|bool;

    /**
     * Set key to hold string value if key does not exist. In that case, it is equal to SET.
     * When key already holds a value, no operation is performed.
     * SETNX is short for "SET if Not eXists".
     *
     * @param  mixed  $key
     * @param  mixed  $value
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function setnx(mixed $key, mixed $value): Cluster|bool;

    /**
     * Overwrites part of the string stored at key, starting at
     * the specified offset, for the entire length of value.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  mixed  $value
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function setrange(mixed $key, int $start, mixed $value): Cluster|int|false;

    /**
     * Returns the members of the set resulting from the intersection of all the given sets.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sinter(mixed $key, mixed ...$other_keys): Cluster|array|false;

    /**
     * Intersect multiple sets and return the cardinality of the result.
     *
     * @param  array  $keys
     * @param  int  $limit
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sintercard(array $keys, int $limit = -1): Cluster|int|false;

    /**
     * This command is equal to SINTER, but instead of returning the resulting set, it is stored in destination.
     * If destination already exists, it is overwritten.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function sinterstore(mixed $key, mixed ...$other_keys): Cluster|int|false;

    /**
     * Returns if `$member` is a member of the set stored at `$key`.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sismember(mixed $key, mixed $member): Cluster|bool;

    /**
     * Interact with the Redis slowlog.
     *
     * @param  array|string  $key_or_address
     * @param  string  $operation
     * @param  mixed  $args,...
     * @return Cluster|array|int|bool
     */
    #[Attributes\RedisCommand]
    public function slowlog(array|string $key_or_address, string $operation, mixed ...$args): Cluster|array|int|bool;

    /**
     * Returns all the members of the set value stored at `$key`.
     *
     * @param  mixed  $key
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function smembers(mixed $key): Cluster|array|false;

    /**
     * Returns whether each member is a member of the set stored at `$key`.
     *
     * @param  mixed  $key
     * @param  mixed  $members,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function smismember(mixed $key, mixed ...$members): Cluster|array|false;

    /**
     * Move member from the set at source to the set at destination.
     *
     * @param  mixed  $srckey
     * @param  mixed  $dstkey
     * @param  mixed  $member
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function smove(mixed $srckey, mixed $dstkey, mixed $member): Cluster|bool;

    /**
     * Sort the elements in a list, set or sorted set.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @return Cluster|array|int|false
     */
    #[Attributes\RedisCommand]
    public function sort(mixed $key, array $options = []): Cluster|array|int|false;

    /**
     * Sort the elements in a list, set or sorted set. Read-only variant of SORT.
     *
     * @param  mixed  $key
     * @param  array  $options
     * @return Cluster|array|int|false
     */
    #[Attributes\RedisCommand]
    public function sort_ro(mixed $key, array $options = []): Cluster|array|int|false;

    /**
     * Removes and returns one or more random members from the set value store at `$key`.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function spop(mixed $key, int $count = 1): mixed;

    /**
     * Returns one or multiple random members from a set.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function srandmember(mixed $key, int $count = 1): mixed;

    /**
     * Remove the specified members from the set stored at `$key`.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @param  mixed  $members,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function srem(mixed $key, mixed $member, mixed ...$members): Cluster|int|false;

    /**
     * Iterates elements of Sets types.
     *
     * @param  mixed  $key
     * @param  mixed  $iterator
     * @param  mixed  $match
     * @param  int  $count
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function sscan(mixed $key, mixed &$iterator, mixed $match = null, int $count = 0): array|false;

    /**
     * Subscribes to the specified shard channels.
     *
     * @param  array  $channels
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function ssubscribe(array $channels, callable $callback): bool;

    /**
     * Returns statistics about Relay.
     *
     * @see \Relay\Relay::stats()
     * @return array
     */
    #[Attributes\Local]
    public static function stats(): array;

    /**
     * Returns the length of the string value stored at `$key`.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function strlen(mixed $key): Cluster|int|false;

    /**
     * Subscribes to the specified channels.
     *
     * @param  array  $channels
     * @param  callable  $callback
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function subscribe(array $channels, callable $callback): bool;

    /**
     * Returns the members of the set resulting from the union of all the given sets.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function sunion(mixed $key, mixed ...$other_keys): Cluster|array|false;

    /**
     * This command is equal to SUNION, but instead of returning the resulting set, it is stored in destination.
     * If destination already exists, it is overwritten.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function sunionstore(mixed $key, mixed ...$other_keys): Cluster|int|false;

    /**
     * Unsubscribes from the given shard channels, or from all of them if none is given.
     *
     * @param  array  $channels
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function sunsubscribe(array $channels = []): bool;

    /**
     * Returns the current time from Redis.
     *
     * @param  array|string  $key_or_address
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function time(array|string $key_or_address): Cluster|array|false;

    /**
     * Alters the last access time of a key(s).
     *
     * @param  array|string  $key_or_array
     * @param  mixed  $more_keys,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function touch(array|string $key_or_array, mixed ...$more_keys): Cluster|int|false;

    /**
     * Returns the remaining time to live of a key that has a timeout in seconds.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function ttl(mixed $key): Cluster|int|false;

    /**
     * Returns the type of a given key.
     *
     * In PhpRedis compatibility mode this will return an integer
     * (one of the REDIS_<type>) constants. Otherwise it will
     * return the string that Redis returns.
     *
     * @param  mixed  $key
     * @return Cluster|int|string|false
     */
    #[Attributes\RedisCommand]
    public function type(mixed $key): Cluster|int|string|bool;

    /**
     * Removes the specified keys without blocking Redis.
     *
     * @param  mixed  $keys,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function unlink(mixed ...$keys): Cluster|int|false;

    /**
     * Unsubscribes from the given channels, or from all of them if none is given.
     *
     * @param  array  $channels
     * @return bool
     */
    #[Attributes\RedisCommand]
    public function unsubscribe(array $channels = []): bool;

    /**
     * Flushes all the previously watched keys for a transaction.
     * If you call EXEC or DISCARD, there's no need to manually call UNWATCH.
     *
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function unwatch(): Cluster|bool;

    /**
     * Marks the given keys to be watched for conditional execution of a transaction.
     *
     * @param  mixed  $key
     * @param  mixed  $other_keys,...
     * @return Cluster|bool
     */
    #[Attributes\RedisCommand]
    public function watch(mixed $key, mixed ...$other_keys): Cluster|bool;

    /**
     * Acknowledge one or more IDs as having been processed by the consumer group.
     *
     * @param  mixed  $key
     * @param  string  $group
     * @param  array  $ids
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function xack(mixed $key, string $group, array $ids): Cluster|int|false;

    /**
     * Append a message to a stream.
     *
     * @param  string  $key
     * @param  string  $id
     * @param  int  $maxlen
     * @param  bool  $approx
     * @param  bool  $nomkstream
     * @return Cluster|string|false
     */
    #[Attributes\RedisCommand]
    public function xadd(mixed $key, string $id, array $values, int $maxlen = 0, bool $approx = false, bool $nomkstream = false): Cluster|string|false;

    /**
     * Automatically take ownership of stream message(s) by metrics
     *
     * @param  string  $key
     * @param  string  $group
     * @param  string  $consumer
     * @param  int  $min_idle
     * @param  string  $start
     * @param  int  $count
     * @param  bool  $justid
     * @return Cluster|array|bool
     */
    #[Attributes\RedisCommand]
    public function xautoclaim(mixed $key, string $group, string $consumer, int $min_idle, string $start, int $count = -1, bool $justid = false): Cluster|bool|array;

    /**
     * Claim ownership of stream message(s).
     *
     * @param  string  $key
     * @param  string  $group
     * @param  string  $consumer
     * @param  int  $min_idle
     * @param  array  $ids
     * @param  array  $options
     * @return Cluster|array|bool
     */
    #[Attributes\RedisCommand]
    public function xclaim(mixed $key, string $group, string $consumer, int $min_idle, array $ids, array $options): Cluster|array|bool;

    /**
     * Remove one or more specific IDs from a stream.
     *
     * @param  string  $key
     * @param  array  $ids
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function xdel(mixed $key, array $ids): Cluster|int|false;

    /**
     * Perform utility operations having to do with consumer groups
     *
     * @param  string  $operation
     * @param  mixed  $key
     * @param  string  $group
     * @param  string  $id_or_consumer
     * @param  bool  $mkstream
     * @param  int  $entries_read
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function xgroup(string $operation, mixed $key = null, string $group = null, string $id_or_consumer = null, bool $mkstream = false, int $entries_read = -2): mixed;

    /**
     * Retrieve information about a stream key.
     *
     * @param  string  $operation
     * @param  string|null  $arg1
     * @param  string|null  $arg2
     * @param  int  $count
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function xinfo(string $operation, string|null $arg1 = null, string|null $arg2 = null, int $count = -1): mixed;

    /**
     * Get the length of a stream.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function xlen(mixed $key): Cluster|int|false;

    /**
     * Query pending entries in a stream.
     *
     * @param  string  $key
     * @param  string  $group
     * @param  string|null  $start
     * @param  string|null  $end
     * @param  int  $count
     * @param  string|null  $consumer
     * @param  int  $idle
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function xpending(mixed $key, string $group, string|null $start = null, string|null $end = null, int $count = -1, string|null $consumer = null, int $idle = 0): Cluster|array|false;

    /**
     * Lists elements in a stream.
     *
     * @param  mixed  $key
     * @param  string  $start
     * @param  string  $end
     * @param  int  $count = -1
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function xrange(mixed $key, string $start, string $end, int $count = -1): Cluster|array|false;

    /**
     * Read messages from a stream.
     *
     * @param  array  $streams
     * @param  int  $count
     * @param  int  $block
     * @return Cluster|array|bool|null
     */
    #[Attributes\RedisCommand]
    public function xread(array $streams, int $count = -1, int $block = -1): Cluster|array|bool|null;

    /**
     * Read messages from a stream using a consumer group.
     *
     * @param  mixed  $key
     * @param  string  $consumer
     * @param  array  $streams
     * @param  int  $count
     * @param  int  $block
     * @return Cluster|array|bool|null
     */
    #[Attributes\RedisCommand]
    public function xreadgroup(mixed $key, string $consumer, array $streams, int $count = 1, int $block = 1): Cluster|array|bool|null;

    /**
     * Get a range of entries from a STREAM ke in reverse chronological order.
     *
     * @param  mixed  $key
     * @param  string  $end
     * @param  string  $start
     * @param  int  $count
     * @return Cluster|array|bool
     */
    #[Attributes\RedisCommand]
    public function xrevrange(mixed $key, string $end, string $start, int $count = -1): Cluster|array|bool;

    /**
     * Truncate a STREAM key in various ways.
     *
     * @param  mixed  $key
     * @param  string  $threshold
     * @param  bool  $approx
     * @param  bool  $minid
     * @param  int  $limit
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function xtrim(mixed $key, string $threshold, bool $approx = false, bool $minid = false, int $limit = -1): Cluster|int|false;

    /**
     * Adds all the specified members with the specified scores to the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $args,...
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function zadd(mixed $key, mixed ...$args): mixed;

    /**
     * Returns the sorted set cardinality (number of elements) of the sorted set stored at key.
     *
     * @param  mixed  $key
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zcard(mixed $key): Cluster|int|false;

    /**
     * Returns the number of elements in the sorted set at key with a score between min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zcount(mixed $key, mixed $min, mixed $max): Cluster|int|false;

    /**
     * This command is similar to ZDIFFSTORE, but instead of storing the
     * resulting sorted set, it is returned to the client.
     *
     * @param  array  $keys
     * @param  array|null  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zdiff(array $keys, array|null $options = null): Cluster|array|false;

    /**
     * Computes the difference between the first and all successive
     * input sorted sets and stores the result in destination.
     *
     * @param  mixed  $dstkey
     * @param  array  $keys
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zdiffstore(mixed $dstkey, array $keys): Cluster|int|false;

    /**
     * Increments the score of member in the sorted set stored at key by increment.
     *
     * @param  mixed  $key
     * @param  float  $score
     * @param  mixed  $member
     * @return Cluster|float|false
     */
    #[Attributes\RedisCommand]
    public function zincrby(mixed $key, float $score, mixed $member): Cluster|float|false;

    /**
     * This command is similar to ZINTERSTORE, but instead of storing
     * the resulting sorted set, it is returned to the client.
     *
     * @param  array  $keys
     * @param  array|null  $weights
     * @param  mixed  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zinter(array $keys, array|null $weights = null, mixed $options = null): Cluster|array|false;

    /**
     * Intersect multiple sorted sets and return the cardinality of the result.
     *
     * @param  array  $keys
     * @param  int  $limit
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zintercard(array $keys, int $limit = -1): Cluster|int|false;

    /**
     * Computes the intersection of numkeys sorted sets given by the
     * specified keys, and stores the result in destination.
     *
     * @param  mixed  $dstkey
     * @param  array  $keys
     * @param  array|null  $weights
     * @param  mixed  $options
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zinterstore(mixed $dstkey, array $keys, array|null $weights = null, mixed $options = null): Cluster|int|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command returns the
     * number of elements in the sorted set at key with a value between min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zlexcount(mixed $key, mixed $min, mixed $max): Cluster|int|false;

    /**
     * Pops one or more elements, that are member-score pairs, from the
     * first non-empty sorted set in the provided list of key names.
     *
     * @param  array  $keys
     * @param  string  $from
     * @param  int  $count
     * @return Cluster|array|null|false
     */
    #[Attributes\RedisCommand]
    public function zmpop(array $keys, string $from, int $count = 1): Cluster|array|null|false;

    /**
     * Returns the scores associated with the specified members in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  mixed  $members,...
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zmscore(mixed $key, mixed ...$members): Cluster|array|false;

    /**
     * Removes and returns up to count members with the highest
     * scores in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zpopmax(mixed $key, int $count = 1): Cluster|array|false;

    /**
     * Removes and returns up to count members with the lowest
     * scores in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  int  $count
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zpopmin(mixed $key, int $count = 1): Cluster|array|false;

    /**
     * When called with just the key argument, return a random element from the sorted set value stored at key.
     * If the provided count argument is positive, return an array of distinct elements.
     *
     * @param  mixed  $key
     * @param  array|null  $options
     * @return mixed
     */
    #[Attributes\RedisCommand]
    public function zrandmember(mixed $key, array|null $options = null): mixed;

    /**
     * Returns the specified range of elements in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  string  $start
     * @param  string  $end
     * @param  mixed  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function zrange(mixed $key, string $start, string $end, mixed $options = null): Cluster|array|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command returns all
     * the elements in the sorted set at key with a value between min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @param  int  $offset
     * @param  int  $count
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zrangebylex(mixed $key, mixed $min, mixed $max, int $offset = -1, int $count = -1): Cluster|array|false;

    /**
     * Returns all the elements in the sorted set at key with a score between
     * min and max (including elements with score equal to min or max).
     *
     * @param  mixed  $key
     * @param  mixed  $start
     * @param  mixed  $end
     * @param  mixed  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function zrangebyscore(mixed $key, mixed $start, mixed $end, mixed $options = null): Cluster|array|false;

    /**
     * Returns all the elements in the sorted set at key with a score between
     * max and min (including elements with score equal to max or min).
     *
     * @param  mixed  $dstkey
     * @param  mixed  $srckey
     * @param  mixed  $start
     * @param  mixed  $end
     * @param  mixed  $options
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zrangestore(mixed $dstkey, mixed $srckey, mixed $start, mixed $end, mixed $options = null): Cluster|int|false;

    /**
     * Returns the rank of member in the sorted set stored at key, with the scores
     * ordered from low to high. The rank (or index) is 0-based, which means
     * that the member with the lowest score has rank 0.
     *
     * @param  mixed  $key
     * @param  mixed  $rank
     * @param  bool  $withscore
     * @return Cluster|array|int|false
     */
    #[Attributes\RedisCommand]
    public function zrank(mixed $key, mixed $rank, bool $withscore = false): Cluster|array|int|false;

    /**
     * Removes the specified members from the sorted set stored at key.
     * Non existing members are ignored.
     *
     * @param  mixed  $key
     * @param  mixed  $args,...
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zrem(mixed $key, mixed ...$args): Cluster|int|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command removes all
     * elements in the sorted set stored at key between the
     * lexicographical range specified by min and max.
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zremrangebylex(mixed $key, mixed $min, mixed $max): Cluster|int|false;

    /**
     * Removes all elements in the sorted set stored at key with rank between
     * start and stop. Both start and stop are 0 -based indexes with 0 being
     * the element with the lowest score.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zremrangebyrank(mixed $key, int $start, int $end): Cluster|int|false;

    /**
     * Removes all elements in the sorted set stored at key with
     * a score between min and max (inclusive).
     *
     * @param  mixed  $key
     * @param  mixed  $min
     * @param  mixed  $max
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zremrangebyscore(mixed $key, mixed $min, mixed $max): Cluster|int|false;

    /**
     * Returns the specified range of elements in the sorted set stored at key.
     *
     * @param  mixed  $key
     * @param  int  $start
     * @param  int  $end
     * @param  mixed  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function zrevrange(mixed $key, int $start, int $end, mixed $options = null): Cluster|array|false;

    /**
     * When all the elements in a sorted set are inserted with the same score,
     * in order to force lexicographical ordering, this command returns all
     * the elements in the sorted set at key with a value between max and min.
     *
     * @param  mixed  $key
     * @param  mixed  $max
     * @param  mixed  $min
     * @param  int  $offset
     * @param  int  $count
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zrevrangebylex(mixed $key, mixed $max, mixed $min, int $offset = -1, int $count = -1): Cluster|array|false;

    /**
     * Returns all the elements in the sorted set at key with a score between
     * max and min (including elements with score equal to max or min).
     *
     * @param  mixed  $key
     * @param  mixed  $start
     * @param  mixed  $end
     * @param  mixed  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand, Attributes\Cached]
    public function zrevrangebyscore(mixed $key, mixed $start, mixed $end, mixed $options = null): Cluster|array|false;

    /**
     * Returns the rank of member in the sorted set stored at key, with the scores
     * ordered from high to low. The rank (or index) is 0-based, which means
     * that the member with the highest score has rank 0.
     *
     * @param  mixed  $key
     * @param  mixed  $rank
     * @param  bool  $withscore
     * @return Cluster|array|int|false
     */
    #[Attributes\RedisCommand]
    public function zrevrank(mixed $key, mixed $rank, bool $withscore = false): Cluster|array|int|false;

    /**
     * Iterates elements of Sorted Set types and their associated scores.
     *
     * @param  mixed  $key
     * @param  mixed  $iterator
     * @param  mixed  $match
     * @param  int  $count
     * @return array|false
     */
    #[Attributes\RedisCommand]
    public function zscan(mixed $key, mixed &$iterator, mixed $match = null, int $count = 0): array|false;

    /**
     * Returns the score of member in the sorted set at key.
     *
     * @param  mixed  $key
     * @param  mixed  $member
     * @return Cluster|float|false
     */
    #[Attributes\RedisCommand]
    public function zscore(mixed $key, mixed $member): Cluster|float|false;

    /**
     * This command is similar to ZUNIONSTORE, but instead of storing
     * the resulting sorted set, it is returned to the client.
     *
     * @param  array  $keys
     * @param  array|null  $weights
     * @param  mixed  $options
     * @return Cluster|array|false
     */
    #[Attributes\RedisCommand]
    public function zunion(array $keys, array|null $weights = null, mixed $options = null): Cluster|array|false;

    /**
     * Computes the union of numkeys sorted sets given by the
     * specified keys, and stores the result in destination.
     *
     * @param  mixed  $dstkey
     * @param  array  $keys
     * @param  array|null  $weights
     * @param  mixed  $options
     * @return Cluster|int|false
     */
    #[Attributes\RedisCommand]
    public function zunionstore(mixed $dstkey, array $keys, array|null $weights = null, mixed $options = null): Cluster|int|false;
}

/**
 * Relay event class.
 */
abstract class Event
{
    /**
     * The integer representing the `INVALIDATED` event.
     *
     * @var int
     */
    public const INVALIDATED = 0;

    /**
     * The integer representing the `INVALIDATED` event.
     *
     * @var int
     */
    public const Invalidated = 0;

    /**
     * The integer representing the `FLUSHED` event.
     *
     * @var int
     */
    public const FLUSHED = 0;

    /**
     * The integer representing the `FLUSHED` event.
     *
     * @var int
     */
    public const Flushed = 0;

    /**
     * The type of the event represented by an integer.
     *
     * @var int
     */
    public int $type;

    /**
     * The event key. Only filled for `INVALIDATED` events.
     *
     * @var mixed
     */
    public mixed $key = null;

    /**
     * Whether the invalidation was created in the client or
     * originated from a Redis PUSH message.
     *
     * @var bool
     */
    public bool $client;
}

namespace Relay\Event;

/**
 * Relay invalidated event class.
 */
final class Invalidated extends \Relay\Event
{
    /**
     * The type of the event represented by an integer.
     *
     * @var int
     */
    public int $type = 1;
}

/**
 * Relay flushed event class.
 */
final class Flushed extends \Relay\Event
{
    /**
     * The type of the event represented by an integer.
     *
     * @var int
     */
    public int $type = 2;
}
